--
-------------------------------------------------------------------------------------------
-- Copyright  2010-2013, Xilinx, Inc.
-- This file contains confidential and proprietary information of Xilinx, Inc. and is
-- protected under U.S. and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by Xilinx, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND XILINX HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) Xilinx shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or Xilinx had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- Xilinx products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of Xilinx products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
-------------------------------------------------------------------------------------------
--
--
-- Definition of a program memory for KCPSM6 including generic parameters for the 
-- convenient selection of device family, program memory size and the ability to include 
-- the JTAG Loader hardware for rapid software development.
--
-- This file is primarily for use during code development and it is recommended that the 
-- appropriate simplified program memory definition be used in a final production design. 
--
--    Generic                  Values             Comments
--    Parameter                Supported
--  
--    C_FAMILY                 "S6"               Spartan-6 device
--                             "V6"               Virtex-6 device
--                             "7S"               7-Series device 
--                                                  (Artix-7, Kintex-7, Virtex-7 or Zynq)
--
--    C_RAM_SIZE_KWORDS        1, 2 or 4          Size of program memory in K-instructions
--
--    C_JTAG_LOADER_ENABLE     0 or 1             Set to '1' to include JTAG Loader
--
-- Notes
--
-- If your design contains MULTIPLE KCPSM6 instances then only one should have the 
-- JTAG Loader enabled at a time (i.e. make sure that C_JTAG_LOADER_ENABLE is only set to 
-- '1' on one instance of the program memory). Advanced users may be interested to know 
-- that it is possible to connect JTAG Loader to multiple memories and then to use the 
-- JTAG Loader utility to specify which memory contents are to be modified. However, 
-- this scheme does require some effort to set up and the additional connectivity of the 
-- multiple BRAMs can impact the placement, routing and performance of the complete 
-- design. Please contact the author at Xilinx for more detailed information. 
--
-- Regardless of the size of program memory specified by C_RAM_SIZE_KWORDS, the complete 
-- 12-bit address bus is connected to KCPSM6. This enables the generic to be modified 
-- without requiring changes to the fundamental hardware definition. However, when the 
-- program memory is 1K then only the lower 10-bits of the address are actually used and 
-- the valid address range is 000 to 3FF hex. Likewise, for a 2K program only the lower 
-- 11-bits of the address are actually used and the valid address range is 000 to 7FF hex.
--
-- Programs are stored in Block Memory (BRAM) and the number of BRAM used depends on the 
-- size of the program and the device family. 
--
-- In a Spartan-6 device a BRAM is capable of holding 1K instructions. Hence a 2K program 
-- will require 2 BRAMs to be used and a 4K program will require 4 BRAMs to be used. It 
-- should be noted that a 4K program is not such a natural fit in a Spartan-6 device and 
-- the implementation also requires a small amount of logic resulting in slightly lower 
-- performance. A Spartan-6 BRAM can also be split into two 9k-bit memories suggesting 
-- that a program containing up to 512 instructions could be implemented. However, there 
-- is a silicon errata which makes this unsuitable and therefore it is not supported by 
-- this file.
--
-- In a Virtex-6 or any 7-Series device a BRAM is capable of holding 2K instructions so 
-- obviously a 2K program requires only a single BRAM. Each BRAM can also be divided into 
-- 2 smaller memories supporting programs of 1K in half of a 36k-bit BRAM (generally 
-- reported as being an 18k-bit BRAM). For a program of 4K instructions, 2 BRAMs are used.
--
--
-- Program defined by '{psmname}.psm'.
--
-- Generated by KCPSM6 Assembler: 2017-11-29T11:56:07. 
--
-- Assembler used ROM_form template: ROM_form_JTAGLoader_14March13.vhd
--
-- Standard IEEE libraries
--
--
package jtag_loader_pkg is
 function addr_width_calc (size_in_k: integer) return integer;
end jtag_loader_pkg;
--
package body jtag_loader_pkg is
  function addr_width_calc (size_in_k: integer) return integer is
   begin
    if (size_in_k = 1) then return 10;
      elsif (size_in_k = 2) then return 11;
      elsif (size_in_k = 4) then return 12;
      else report "Invalid BlockRAM size. Please set to 1, 2 or 4 K words." severity FAILURE;
    end if;
    return 0;
  end function addr_width_calc;
end package body;
--
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use work.jtag_loader_pkg.ALL;
--
-- The Unisim Library is used to define Xilinx primitives. It is also used during
-- simulation. The source can be viewed at %XILINX%\vhdl\src\unisims\unisim_VCOMP.vhd
--  
library unisim;
use unisim.vcomponents.all;
--
--
entity m25p16_spi_uart_bridge is
  generic(             C_FAMILY : string := "S6"; 
              C_RAM_SIZE_KWORDS : integer := 1;
           C_JTAG_LOADER_ENABLE : integer := 0);
  Port (      address : in std_logic_vector(11 downto 0);
          instruction : out std_logic_vector(17 downto 0);
               enable : in std_logic;
                  rdl : out std_logic;                    
                  clk : in std_logic);
  end m25p16_spi_uart_bridge;
--
architecture low_level_definition of m25p16_spi_uart_bridge is
--
signal       address_a : std_logic_vector(15 downto 0);
signal        pipe_a11 : std_logic;
signal       data_in_a : std_logic_vector(35 downto 0);
signal      data_out_a : std_logic_vector(35 downto 0);
signal    data_out_a_l : std_logic_vector(35 downto 0);
signal    data_out_a_h : std_logic_vector(35 downto 0);
signal   data_out_a_ll : std_logic_vector(35 downto 0);
signal   data_out_a_lh : std_logic_vector(35 downto 0);
signal   data_out_a_hl : std_logic_vector(35 downto 0);
signal   data_out_a_hh : std_logic_vector(35 downto 0);
signal       address_b : std_logic_vector(15 downto 0);
signal       data_in_b : std_logic_vector(35 downto 0);
signal     data_in_b_l : std_logic_vector(35 downto 0);
signal    data_in_b_ll : std_logic_vector(35 downto 0);
signal    data_in_b_hl : std_logic_vector(35 downto 0);
signal      data_out_b : std_logic_vector(35 downto 0);
signal    data_out_b_l : std_logic_vector(35 downto 0);
signal   data_out_b_ll : std_logic_vector(35 downto 0);
signal   data_out_b_hl : std_logic_vector(35 downto 0);
signal     data_in_b_h : std_logic_vector(35 downto 0);
signal    data_in_b_lh : std_logic_vector(35 downto 0);
signal    data_in_b_hh : std_logic_vector(35 downto 0);
signal    data_out_b_h : std_logic_vector(35 downto 0);
signal   data_out_b_lh : std_logic_vector(35 downto 0);
signal   data_out_b_hh : std_logic_vector(35 downto 0);
signal        enable_b : std_logic;
signal           clk_b : std_logic;
signal            we_b : std_logic_vector(7 downto 0);
signal          we_b_l : std_logic_vector(3 downto 0);
signal          we_b_h : std_logic_vector(3 downto 0);
-- 
signal       jtag_addr : std_logic_vector(11 downto 0);
signal         jtag_we : std_logic;
signal       jtag_we_l : std_logic;
signal       jtag_we_h : std_logic;
signal        jtag_clk : std_logic;
signal        jtag_din : std_logic_vector(17 downto 0);
signal       jtag_dout : std_logic_vector(17 downto 0);
signal     jtag_dout_1 : std_logic_vector(17 downto 0);
signal         jtag_en : std_logic_vector(0 downto 0);
-- 
signal picoblaze_reset : std_logic_vector(0 downto 0);
signal         rdl_bus : std_logic_vector(0 downto 0);
--
constant BRAM_ADDRESS_WIDTH  : integer := addr_width_calc(C_RAM_SIZE_KWORDS);
--
--
component jtag_loader_6
generic(                C_JTAG_LOADER_ENABLE : integer := 1;
                                    C_FAMILY : string  := "V6";
                             C_NUM_PICOBLAZE : integer := 1;
                       C_BRAM_MAX_ADDR_WIDTH : integer := 10;
          C_PICOBLAZE_INSTRUCTION_DATA_WIDTH : integer := 18;
                                C_JTAG_CHAIN : integer := 2;
                              C_ADDR_WIDTH_0 : integer := 10;
                              C_ADDR_WIDTH_1 : integer := 10;
                              C_ADDR_WIDTH_2 : integer := 10;
                              C_ADDR_WIDTH_3 : integer := 10;
                              C_ADDR_WIDTH_4 : integer := 10;
                              C_ADDR_WIDTH_5 : integer := 10;
                              C_ADDR_WIDTH_6 : integer := 10;
                              C_ADDR_WIDTH_7 : integer := 10);
port(              picoblaze_reset : out std_logic_vector(C_NUM_PICOBLAZE-1 downto 0);
                           jtag_en : out std_logic_vector(C_NUM_PICOBLAZE-1 downto 0);
                          jtag_din : out STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                         jtag_addr : out STD_LOGIC_VECTOR(C_BRAM_MAX_ADDR_WIDTH-1 downto 0);
                          jtag_clk : out std_logic;
                           jtag_we : out std_logic;
                       jtag_dout_0 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_1 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_2 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_3 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_4 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_5 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_6 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_7 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0));
end component;
--
begin
  --
  --  
  ram_1k_generate : if (C_RAM_SIZE_KWORDS = 1) generate
 
    s6: if (C_FAMILY = "S6") generate 
      --
      address_a(13 downto 0) <= address(9 downto 0) & "0000";
      instruction <= data_out_a(33 downto 32) & data_out_a(15 downto 0);
      data_in_a <= "0000000000000000000000000000000000" & address(11 downto 10);
      jtag_dout <= data_out_b(33 downto 32) & data_out_b(15 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b <= "00" & data_out_b(33 downto 32) & "0000000000000000" & data_out_b(15 downto 0);
        address_b(13 downto 0) <= "00000000000000";
        we_b(3 downto 0) <= "0000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b <= "00" & jtag_din(17 downto 16) & "0000000000000000" & jtag_din(15 downto 0);
        address_b(13 downto 0) <= jtag_addr(9 downto 0) & "0000";
        we_b(3 downto 0) <= jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      --
      kcpsm6_rom: RAMB16BWER
      generic map ( DATA_WIDTH_A => 18,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 18,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"6011D04090001700180019072009200AB001B03171FF0035B6740031B1740035",
                    INIT_01 => X"35DFD000D57B9000D561201900D92009D1FF700061107001038B009903C203E4",
                    INIT_02 => X"1010112712005000150A500095F690001507E02B9511900095E9900015B95000",
                    INIT_03 => X"004250006039B200B10090010000203910801196129820391040110D12032039",
                    INIT_04 => X"004302800043029000431200B654500092046044D2019205D208B6645000B674",
                    INIT_05 => X"5804191F005F191E50000620006107200043004938FB2061004938FB50000270",
                    INIT_06 => X"12A8004312C000431280004312040043120000431200B6545000B67400430049",
                    INIT_07 => X"12080043120000431200004312FF004312FF004312FF00431201004312010043",
                    INIT_08 => X"120200431201004312A8004312C0004312030043120200431201004312DC0043",
                    INIT_09 => X"D280B2005000D20400521800193950006093C67016010043A2602106B6740043",
                    INIT_0A => X"005FB71318081900B674009317131600004312040043120000431200B6546064",
                    INIT_0B => X"16F41701191918000106B6740093171C16160049190CB6540123B615B7141904",
                    INIT_0C => X"002D17101901500061101000D1FFB11C005FD701B726191D005F1704191B0123",
                    INIT_0D => X"2128D20020EFD214005218281903205F002D170819010035B6740031B274205F",
                    INIT_0E => X"1903212DD21320CDD21D20CDD21B20CDD21A20CDD21820D5D21C182C2138D217",
                    INIT_0F => X"D21D20CDD21B20CDD21A20CDD21820D1D21C180C22FED2002346D21700521808",
                    INIT_10 => X"005C1868005C18481700005C1828005C18081708500022F7D215230ED21320CD",
                    INIT_11 => X"7100005F1701190000CD182800CD1808005C18E8005C18C8005C18A8005C1888",
                    INIT_12 => X"1702182C1901202D005F1701182C190120610260004300495804165017001904",
                    INIT_13 => X"DC0470006FF06CC0700100551828192650007F007E007D007C007B007A00005F",
                    INIT_14 => X"0270B6540055182819280B700A601000F700D6002179F700D60073006146FFFF",
                    INIT_15 => X"2162FFFFDC04350014011301E230004314001500130000431238004302600043",
                    INIT_16 => X"0123182C192827900680B6740950084063306158E5B0C4A02167D3FF2167D20A",
                    INIT_17 => X"6BB06AA06990688067706330700150007000DFFF4187DFFF7001005F17401901",
                    INIT_18 => X"B401B500500013001F01418EDF0001D0DF006BA01000FB00DA0070006FF06CC0",
                    INIT_19 => X"69906880677061B060A050001A2A1B021C005000DD801FFE619BF40AF50DD302",
                    INIT_1A => X"61B5C250A54021B4D20003B9190718001740141561B4DD8061B502C91AE21B02",
                    INIT_1B => X"1A07170050001E0070016990688067706B106A0070001D002197E1AA14011701",
                    INIT_1C => X"500061C1CAB01A01475061C69001490048004706100490000021001BA5A01B0D",
                    INIT_1D => X"17001800190661E3D220B20561FB02C91AD41B021C000D2003B9170018001907",
                    INIT_1E => X"D27350001A301B021C0361EFD26521E01C0261E9D262A19701BE50001A341B02",
                    INIT_1F => X"01BE619702C91ADA1B0250001A261B021C0361E0D27250001A341B021C0561F5",
                    INIT_20 => X"FE1FDDFF3E001D017001002D6213D30070006330700150001A051B021C04A197",
                    INIT_21 => X"DE0F2222CE301E01700039003800170103D370016240A4E01E00500022347000",
                    INIT_22 => X"22347000170018FE190670014BA01A271B02023450001E001D0003F622142214",
                    INIT_23 => X"6D706B806A9070010055182819242254DC0070006CC07001700003F603E47001",
                    INIT_24 => X"1D006CC067D068B069A002BA182C02AB193470001440151F1C00170018FF1906",
                    INIT_25 => X"182C02AB70001440151F700119342263DC050055182819242261DC032261DC04",
                    INIT_26 => X"6275D902227400CD00D56274D100180C0180626ED9010290700120D5182C02BA",
                    INIT_27 => X"F702F800387F2286D700F80003B014091300F800588018007001037D70006380",
                    INIT_28 => X"F20C9207F20B00521903F20A005218081900F109700103AB7000F80003B01300",
                    INIT_29 => X"00430270B65470006820225F70006294D80E1801029BA28079341800F20D9206",
                    INIT_2A => X"39003800170103B970015000700137001601B674004302800043029000430260",
                    INIT_2B => X"1903005F17201901012319245000700062ACB5009401029B22B8D20022B4DC02",
                    INIT_2C => X"5000C560A6401000D5004BA01400500062BFF720D600005519205000D2170052",
                    INIT_2D => X"152F152015541553154F15501500152F152015541545154722CA3B001A011401",
                    INIT_2E => X"153A156E156F156915741561157A15691572156F15681574157515411500155F",
                    INIT_2F => X"6110700150007000FE03DDFF3E001D0170011500156315691573156115421520",
                    INIT_30 => X"190102F7205F180C1901C05FD201170100521800191D602D02F71000D1007000",
                    INIT_31 => X"15001780180019077001005518081924005F1702202D005F700067107001180C",
                    INIT_32 => X"17031800191D9000F700D65B00551808192622BA180C02AB191470001C021440",
                    INIT_33 => X"22BA180C02AB191470001C02141E150017C0180019077001005518081924005F",
                    INIT_34 => X"0055180819262327D2022316D20100521800191D205F172219011000D2009203",
                    INIT_35 => X"00430270B65464606570005518081928677066605000D2029205A340F710D600",
                    INIT_36 => X"7001B674636BD2079205D7107000677066601710160070010043121800430260",
                    INIT_37 => X"00431200B634205F174019010123180C19282750064070006770666065506440",
                    INIT_38 => X"B31DB2000031037D130023ABB674004352020230004312000043126000431200",
                    INIT_39 => X"B33903B01426131D03B01436133103B01431132C03B0142C132763ABF30ED280",
                    INIT_3A => X"5000B6740031B6F40031D340B33AD320B33BD3E0B336D3C0B337D3A0B338D380",
                    INIT_3B => X"02700042028000420290004212035000B67463B1C3401301E2300043A230B634",
                    INIT_3C => X"130123D1D3FFE2300042130000420270004202800042029000421203203F0042",
                    INIT_3D => X"03F1003F02D00042027000420280004202900042120203F40D205000B67423CB",
                    INIT_3E => X"63EDD20103F1003F02700042028000420290004212D803F4500003F663DFD201",
                    INIT_3F => X"00000000000000000000000000000000203F1204203F1206203F004212055000",
                   INITP_00 => X"22222222222222AA0888A288888A30AAAD52020202677776376B77AAC00AA6AA",
                   INITP_01 => X"77774DD83777774DDD8282AA8274D08200A8228080A0888B4D82D62A22222222",
                   INITP_02 => X"555FDDE0816075DDD55A022228035D5D75E0955602808A006088888888220B77",
                   INITP_03 => X"B8203603603637A0034E0080B53553A008D55CB5D3600DE0556034D508776D75",
                   INITP_04 => X"A36820EDDAB434E22C3360DD15623000578377EBB038282B35D5B42F55EDDE03",
                   INITP_05 => X"7B15EAAAAAAAAAAAAAAAAA95D362D636208BD6DD56ED68888B6F58488A282EE8",
                   INITP_06 => X"8A081755EC2D438889605C358376083488C003820358223000E08ADC283483B7",
                   INITP_07 => X"000088A2CA2222ACA222222A7688888A2222B5A2AAA22222208208350A2A0888")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a(31 downto 0),
                  DOPA => data_out_a(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b(31 downto 0),
                  DOPB => data_out_b(35 downto 32), 
                   DIB => data_in_b(31 downto 0),
                  DIPB => data_in_b(35 downto 32), 
                   WEB => we_b(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
    --               
    end generate s6;
    --
    --
    v6 : if (C_FAMILY = "V6") generate
      --
      address_a(13 downto 0) <= address(9 downto 0) & "1111";
      instruction <= data_out_a(17 downto 0);
      data_in_a(17 downto 0) <= "0000000000000000" & address(11 downto 10);
      jtag_dout <= data_out_b(17 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b(17 downto 0) <= data_out_b(17 downto 0);
        address_b(13 downto 0) <= "11111111111111";
        we_b(3 downto 0) <= "0000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b(17 downto 0) <= jtag_din(17 downto 0);
        address_b(13 downto 0) <= jtag_addr(9 downto 0) & "1111";
        we_b(3 downto 0) <= jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      -- 
      kcpsm6_rom: RAMB18E1
      generic map ( READ_WIDTH_A => 18,
                    WRITE_WIDTH_A => 18,
                    DOA_REG => 0,
                    INIT_A => "000000000000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 18,
                    WRITE_WIDTH_B => 18,
                    DOB_REG => 0,
                    INIT_B => X"000000000000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    SIM_DEVICE => "VIRTEX6",
                    INIT_00 => X"6011D04090001700180019072009200AB001B03171FF0035B6740031B1740035",
                    INIT_01 => X"35DFD000D57B9000D561201900D92009D1FF700061107001038B009903C203E4",
                    INIT_02 => X"1010112712005000150A500095F690001507E02B9511900095E9900015B95000",
                    INIT_03 => X"004250006039B200B10090010000203910801196129820391040110D12032039",
                    INIT_04 => X"004302800043029000431200B654500092046044D2019205D208B6645000B674",
                    INIT_05 => X"5804191F005F191E50000620006107200043004938FB2061004938FB50000270",
                    INIT_06 => X"12A8004312C000431280004312040043120000431200B6545000B67400430049",
                    INIT_07 => X"12080043120000431200004312FF004312FF004312FF00431201004312010043",
                    INIT_08 => X"120200431201004312A8004312C0004312030043120200431201004312DC0043",
                    INIT_09 => X"D280B2005000D20400521800193950006093C67016010043A2602106B6740043",
                    INIT_0A => X"005FB71318081900B674009317131600004312040043120000431200B6546064",
                    INIT_0B => X"16F41701191918000106B6740093171C16160049190CB6540123B615B7141904",
                    INIT_0C => X"002D17101901500061101000D1FFB11C005FD701B726191D005F1704191B0123",
                    INIT_0D => X"2128D20020EFD214005218281903205F002D170819010035B6740031B274205F",
                    INIT_0E => X"1903212DD21320CDD21D20CDD21B20CDD21A20CDD21820D5D21C182C2138D217",
                    INIT_0F => X"D21D20CDD21B20CDD21A20CDD21820D1D21C180C22FED2002346D21700521808",
                    INIT_10 => X"005C1868005C18481700005C1828005C18081708500022F7D215230ED21320CD",
                    INIT_11 => X"7100005F1701190000CD182800CD1808005C18E8005C18C8005C18A8005C1888",
                    INIT_12 => X"1702182C1901202D005F1701182C190120610260004300495804165017001904",
                    INIT_13 => X"DC0470006FF06CC0700100551828192650007F007E007D007C007B007A00005F",
                    INIT_14 => X"0270B6540055182819280B700A601000F700D6002179F700D60073006146FFFF",
                    INIT_15 => X"2162FFFFDC04350014011301E230004314001500130000431238004302600043",
                    INIT_16 => X"0123182C192827900680B6740950084063306158E5B0C4A02167D3FF2167D20A",
                    INIT_17 => X"6BB06AA06990688067706330700150007000DFFF4187DFFF7001005F17401901",
                    INIT_18 => X"B401B500500013001F01418EDF0001D0DF006BA01000FB00DA0070006FF06CC0",
                    INIT_19 => X"69906880677061B060A050001A2A1B021C005000DD801FFE619BF40AF50DD302",
                    INIT_1A => X"61B5C250A54021B4D20003B9190718001740141561B4DD8061B502C91AE21B02",
                    INIT_1B => X"1A07170050001E0070016990688067706B106A0070001D002197E1AA14011701",
                    INIT_1C => X"500061C1CAB01A01475061C69001490048004706100490000021001BA5A01B0D",
                    INIT_1D => X"17001800190661E3D220B20561FB02C91AD41B021C000D2003B9170018001907",
                    INIT_1E => X"D27350001A301B021C0361EFD26521E01C0261E9D262A19701BE50001A341B02",
                    INIT_1F => X"01BE619702C91ADA1B0250001A261B021C0361E0D27250001A341B021C0561F5",
                    INIT_20 => X"FE1FDDFF3E001D017001002D6213D30070006330700150001A051B021C04A197",
                    INIT_21 => X"DE0F2222CE301E01700039003800170103D370016240A4E01E00500022347000",
                    INIT_22 => X"22347000170018FE190670014BA01A271B02023450001E001D0003F622142214",
                    INIT_23 => X"6D706B806A9070010055182819242254DC0070006CC07001700003F603E47001",
                    INIT_24 => X"1D006CC067D068B069A002BA182C02AB193470001440151F1C00170018FF1906",
                    INIT_25 => X"182C02AB70001440151F700119342263DC050055182819242261DC032261DC04",
                    INIT_26 => X"6275D902227400CD00D56274D100180C0180626ED9010290700120D5182C02BA",
                    INIT_27 => X"F702F800387F2286D700F80003B014091300F800588018007001037D70006380",
                    INIT_28 => X"F20C9207F20B00521903F20A005218081900F109700103AB7000F80003B01300",
                    INIT_29 => X"00430270B65470006820225F70006294D80E1801029BA28079341800F20D9206",
                    INIT_2A => X"39003800170103B970015000700137001601B674004302800043029000430260",
                    INIT_2B => X"1903005F17201901012319245000700062ACB5009401029B22B8D20022B4DC02",
                    INIT_2C => X"5000C560A6401000D5004BA01400500062BFF720D600005519205000D2170052",
                    INIT_2D => X"152F152015541553154F15501500152F152015541545154722CA3B001A011401",
                    INIT_2E => X"153A156E156F156915741561157A15691572156F15681574157515411500155F",
                    INIT_2F => X"6110700150007000FE03DDFF3E001D0170011500156315691573156115421520",
                    INIT_30 => X"190102F7205F180C1901C05FD201170100521800191D602D02F71000D1007000",
                    INIT_31 => X"15001780180019077001005518081924005F1702202D005F700067107001180C",
                    INIT_32 => X"17031800191D9000F700D65B00551808192622BA180C02AB191470001C021440",
                    INIT_33 => X"22BA180C02AB191470001C02141E150017C0180019077001005518081924005F",
                    INIT_34 => X"0055180819262327D2022316D20100521800191D205F172219011000D2009203",
                    INIT_35 => X"00430270B65464606570005518081928677066605000D2029205A340F710D600",
                    INIT_36 => X"7001B674636BD2079205D7107000677066601710160070010043121800430260",
                    INIT_37 => X"00431200B634205F174019010123180C19282750064070006770666065506440",
                    INIT_38 => X"B31DB2000031037D130023ABB674004352020230004312000043126000431200",
                    INIT_39 => X"B33903B01426131D03B01436133103B01431132C03B0142C132763ABF30ED280",
                    INIT_3A => X"5000B6740031B6F40031D340B33AD320B33BD3E0B336D3C0B337D3A0B338D380",
                    INIT_3B => X"02700042028000420290004212035000B67463B1C3401301E2300043A230B634",
                    INIT_3C => X"130123D1D3FFE2300042130000420270004202800042029000421203203F0042",
                    INIT_3D => X"03F1003F02D00042027000420280004202900042120203F40D205000B67423CB",
                    INIT_3E => X"63EDD20103F1003F02700042028000420290004212D803F4500003F663DFD201",
                    INIT_3F => X"00000000000000000000000000000000203F1204203F1206203F004212055000",
                   INITP_00 => X"22222222222222AA0888A288888A30AAAD52020202677776376B77AAC00AA6AA",
                   INITP_01 => X"77774DD83777774DDD8282AA8274D08200A8228080A0888B4D82D62A22222222",
                   INITP_02 => X"555FDDE0816075DDD55A022228035D5D75E0955602808A006088888888220B77",
                   INITP_03 => X"B8203603603637A0034E0080B53553A008D55CB5D3600DE0556034D508776D75",
                   INITP_04 => X"A36820EDDAB434E22C3360DD15623000578377EBB038282B35D5B42F55EDDE03",
                   INITP_05 => X"7B15EAAAAAAAAAAAAAAAAA95D362D636208BD6DD56ED68888B6F58488A282EE8",
                   INITP_06 => X"8A081755EC2D438889605C358376083488C003820358223000E08ADC283483B7",
                   INITP_07 => X"000088A2CA2222ACA222222A7688888A2222B5A2AAA22222208208350A2A0888")
      port map(   ADDRARDADDR => address_a(13 downto 0),
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a(15 downto 0),
                      DOPADOP => data_out_a(17 downto 16), 
                        DIADI => data_in_a(15 downto 0),
                      DIPADIP => data_in_a(17 downto 16), 
                          WEA => "00",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b(13 downto 0),
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b(15 downto 0),
                      DOPBDOP => data_out_b(17 downto 16), 
                        DIBDI => data_in_b(15 downto 0),
                      DIPBDIP => data_in_b(17 downto 16), 
                        WEBWE => we_b(3 downto 0),
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0');
      --
    end generate v6;
    --
    --
    akv7 : if (C_FAMILY = "7S") generate
      --
      address_a(13 downto 0) <= address(9 downto 0) & "1111";
      instruction <= data_out_a(17 downto 0);
      data_in_a(17 downto 0) <= "0000000000000000" & address(11 downto 10);
      jtag_dout <= data_out_b(17 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b(17 downto 0) <= data_out_b(17 downto 0);
        address_b(13 downto 0) <= "11111111111111";
        we_b(3 downto 0) <= "0000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b(17 downto 0) <= jtag_din(17 downto 0);
        address_b(13 downto 0) <= jtag_addr(9 downto 0) & "1111";
        we_b(3 downto 0) <= jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      -- 
      kcpsm6_rom: RAMB18E1
      generic map ( READ_WIDTH_A => 18,
                    WRITE_WIDTH_A => 18,
                    DOA_REG => 0,
                    INIT_A => "000000000000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 18,
                    WRITE_WIDTH_B => 18,
                    DOB_REG => 0,
                    INIT_B => X"000000000000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    SIM_DEVICE => "7SERIES",
                    INIT_00 => X"6011D04090001700180019072009200AB001B03171FF0035B6740031B1740035",
                    INIT_01 => X"35DFD000D57B9000D561201900D92009D1FF700061107001038B009903C203E4",
                    INIT_02 => X"1010112712005000150A500095F690001507E02B9511900095E9900015B95000",
                    INIT_03 => X"004250006039B200B10090010000203910801196129820391040110D12032039",
                    INIT_04 => X"004302800043029000431200B654500092046044D2019205D208B6645000B674",
                    INIT_05 => X"5804191F005F191E50000620006107200043004938FB2061004938FB50000270",
                    INIT_06 => X"12A8004312C000431280004312040043120000431200B6545000B67400430049",
                    INIT_07 => X"12080043120000431200004312FF004312FF004312FF00431201004312010043",
                    INIT_08 => X"120200431201004312A8004312C0004312030043120200431201004312DC0043",
                    INIT_09 => X"D280B2005000D20400521800193950006093C67016010043A2602106B6740043",
                    INIT_0A => X"005FB71318081900B674009317131600004312040043120000431200B6546064",
                    INIT_0B => X"16F41701191918000106B6740093171C16160049190CB6540123B615B7141904",
                    INIT_0C => X"002D17101901500061101000D1FFB11C005FD701B726191D005F1704191B0123",
                    INIT_0D => X"2128D20020EFD214005218281903205F002D170819010035B6740031B274205F",
                    INIT_0E => X"1903212DD21320CDD21D20CDD21B20CDD21A20CDD21820D5D21C182C2138D217",
                    INIT_0F => X"D21D20CDD21B20CDD21A20CDD21820D1D21C180C22FED2002346D21700521808",
                    INIT_10 => X"005C1868005C18481700005C1828005C18081708500022F7D215230ED21320CD",
                    INIT_11 => X"7100005F1701190000CD182800CD1808005C18E8005C18C8005C18A8005C1888",
                    INIT_12 => X"1702182C1901202D005F1701182C190120610260004300495804165017001904",
                    INIT_13 => X"DC0470006FF06CC0700100551828192650007F007E007D007C007B007A00005F",
                    INIT_14 => X"0270B6540055182819280B700A601000F700D6002179F700D60073006146FFFF",
                    INIT_15 => X"2162FFFFDC04350014011301E230004314001500130000431238004302600043",
                    INIT_16 => X"0123182C192827900680B6740950084063306158E5B0C4A02167D3FF2167D20A",
                    INIT_17 => X"6BB06AA06990688067706330700150007000DFFF4187DFFF7001005F17401901",
                    INIT_18 => X"B401B500500013001F01418EDF0001D0DF006BA01000FB00DA0070006FF06CC0",
                    INIT_19 => X"69906880677061B060A050001A2A1B021C005000DD801FFE619BF40AF50DD302",
                    INIT_1A => X"61B5C250A54021B4D20003B9190718001740141561B4DD8061B502C91AE21B02",
                    INIT_1B => X"1A07170050001E0070016990688067706B106A0070001D002197E1AA14011701",
                    INIT_1C => X"500061C1CAB01A01475061C69001490048004706100490000021001BA5A01B0D",
                    INIT_1D => X"17001800190661E3D220B20561FB02C91AD41B021C000D2003B9170018001907",
                    INIT_1E => X"D27350001A301B021C0361EFD26521E01C0261E9D262A19701BE50001A341B02",
                    INIT_1F => X"01BE619702C91ADA1B0250001A261B021C0361E0D27250001A341B021C0561F5",
                    INIT_20 => X"FE1FDDFF3E001D017001002D6213D30070006330700150001A051B021C04A197",
                    INIT_21 => X"DE0F2222CE301E01700039003800170103D370016240A4E01E00500022347000",
                    INIT_22 => X"22347000170018FE190670014BA01A271B02023450001E001D0003F622142214",
                    INIT_23 => X"6D706B806A9070010055182819242254DC0070006CC07001700003F603E47001",
                    INIT_24 => X"1D006CC067D068B069A002BA182C02AB193470001440151F1C00170018FF1906",
                    INIT_25 => X"182C02AB70001440151F700119342263DC050055182819242261DC032261DC04",
                    INIT_26 => X"6275D902227400CD00D56274D100180C0180626ED9010290700120D5182C02BA",
                    INIT_27 => X"F702F800387F2286D700F80003B014091300F800588018007001037D70006380",
                    INIT_28 => X"F20C9207F20B00521903F20A005218081900F109700103AB7000F80003B01300",
                    INIT_29 => X"00430270B65470006820225F70006294D80E1801029BA28079341800F20D9206",
                    INIT_2A => X"39003800170103B970015000700137001601B674004302800043029000430260",
                    INIT_2B => X"1903005F17201901012319245000700062ACB5009401029B22B8D20022B4DC02",
                    INIT_2C => X"5000C560A6401000D5004BA01400500062BFF720D600005519205000D2170052",
                    INIT_2D => X"152F152015541553154F15501500152F152015541545154722CA3B001A011401",
                    INIT_2E => X"153A156E156F156915741561157A15691572156F15681574157515411500155F",
                    INIT_2F => X"6110700150007000FE03DDFF3E001D0170011500156315691573156115421520",
                    INIT_30 => X"190102F7205F180C1901C05FD201170100521800191D602D02F71000D1007000",
                    INIT_31 => X"15001780180019077001005518081924005F1702202D005F700067107001180C",
                    INIT_32 => X"17031800191D9000F700D65B00551808192622BA180C02AB191470001C021440",
                    INIT_33 => X"22BA180C02AB191470001C02141E150017C0180019077001005518081924005F",
                    INIT_34 => X"0055180819262327D2022316D20100521800191D205F172219011000D2009203",
                    INIT_35 => X"00430270B65464606570005518081928677066605000D2029205A340F710D600",
                    INIT_36 => X"7001B674636BD2079205D7107000677066601710160070010043121800430260",
                    INIT_37 => X"00431200B634205F174019010123180C19282750064070006770666065506440",
                    INIT_38 => X"B31DB2000031037D130023ABB674004352020230004312000043126000431200",
                    INIT_39 => X"B33903B01426131D03B01436133103B01431132C03B0142C132763ABF30ED280",
                    INIT_3A => X"5000B6740031B6F40031D340B33AD320B33BD3E0B336D3C0B337D3A0B338D380",
                    INIT_3B => X"02700042028000420290004212035000B67463B1C3401301E2300043A230B634",
                    INIT_3C => X"130123D1D3FFE2300042130000420270004202800042029000421203203F0042",
                    INIT_3D => X"03F1003F02D00042027000420280004202900042120203F40D205000B67423CB",
                    INIT_3E => X"63EDD20103F1003F02700042028000420290004212D803F4500003F663DFD201",
                    INIT_3F => X"00000000000000000000000000000000203F1204203F1206203F004212055000",
                   INITP_00 => X"22222222222222AA0888A288888A30AAAD52020202677776376B77AAC00AA6AA",
                   INITP_01 => X"77774DD83777774DDD8282AA8274D08200A8228080A0888B4D82D62A22222222",
                   INITP_02 => X"555FDDE0816075DDD55A022228035D5D75E0955602808A006088888888220B77",
                   INITP_03 => X"B8203603603637A0034E0080B53553A008D55CB5D3600DE0556034D508776D75",
                   INITP_04 => X"A36820EDDAB434E22C3360DD15623000578377EBB038282B35D5B42F55EDDE03",
                   INITP_05 => X"7B15EAAAAAAAAAAAAAAAAA95D362D636208BD6DD56ED68888B6F58488A282EE8",
                   INITP_06 => X"8A081755EC2D438889605C358376083488C003820358223000E08ADC283483B7",
                   INITP_07 => X"000088A2CA2222ACA222222A7688888A2222B5A2AAA22222208208350A2A0888")
      port map(   ADDRARDADDR => address_a(13 downto 0),
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a(15 downto 0),
                      DOPADOP => data_out_a(17 downto 16), 
                        DIADI => data_in_a(15 downto 0),
                      DIPADIP => data_in_a(17 downto 16), 
                          WEA => "00",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b(13 downto 0),
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b(15 downto 0),
                      DOPBDOP => data_out_b(17 downto 16), 
                        DIBDI => data_in_b(15 downto 0),
                      DIPBDIP => data_in_b(17 downto 16), 
                        WEBWE => we_b(3 downto 0),
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0');
      --
    end generate akv7;
    --
  end generate ram_1k_generate;
  --
  --
  --
  ram_2k_generate : if (C_RAM_SIZE_KWORDS = 2) generate
    --
    --
    s6: if (C_FAMILY = "S6") generate
      --
      address_a(13 downto 0) <= address(10 downto 0) & "000";
      instruction <= data_out_a_h(32) & data_out_a_h(7 downto 0) & data_out_a_l(32) & data_out_a_l(7 downto 0);
      data_in_a <= "00000000000000000000000000000000000" & address(11);
      jtag_dout <= data_out_b_h(32) & data_out_b_h(7 downto 0) & data_out_b_l(32) & data_out_b_l(7 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b_l <= "000" & data_out_b_l(32) & "000000000000000000000000" & data_out_b_l(7 downto 0);
        data_in_b_h <= "000" & data_out_b_h(32) & "000000000000000000000000" & data_out_b_h(7 downto 0);
        address_b(13 downto 0) <= "00000000000000";
        we_b(3 downto 0) <= "0000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b_h <= "000" & jtag_din(17) & "000000000000000000000000" & jtag_din(16 downto 9);
        data_in_b_l <= "000" & jtag_din(8) & "000000000000000000000000" & jtag_din(7 downto 0);
        address_b(13 downto 0) <= jtag_addr(10 downto 0) & "000";
        we_b(3 downto 0) <= jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      --
      kcpsm6_rom_l: RAMB16BWER
      generic map ( DATA_WIDTH_A => 9,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 9,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"DF007B006119D909FF0010018B99C2E4114000000007090A0131FF3574317435",
                    INIT_01 => X"420039000001003980969839400D0339102700000A00F600072B1100E900B900",
                    INIT_02 => X"041F5F1E002061204349FB6149FB007043804390430054000444010508640074",
                    INIT_03 => X"084300430043FF43FF43FF4301430143A843C043804304430043005400744349",
                    INIT_04 => X"8000000452003900937001436006744302430143A843C043034302430143DC43",
                    INIT_05 => X"F40119000674931C16490C54231514045F130800749313004304430043005464",
                    INIT_06 => X"2800EF145228035F2D0801357431745F2D1001001000FF1C5F01261D5F041B23",
                    INIT_07 => X"1DCD1BCD1ACD18D11C0CFE0046175208032D13CD1DCD1BCD1ACD18D51C2C3817",
                    INIT_08 => X"005F0100CD28CD085CE85CC85CA85C885C685C48005C285C080800F7150E13CD",
                    INIT_09 => X"0400F0C001552826000000000000005F022C012D5F012C016160434904500004",
                    INIT_0A => X"62FF0400010130430000004338436043705455282870600000007900000046FF",
                    INIT_0B => X"B0A090807030010000FF87FF015F4001232C2890807450403058B0A067FF670A",
                    INIT_0C => X"908070B0A0002A02000080FE9B0A0D0201000000018E00D000A000000000F0C0",
                    INIT_0D => X"07000000019080701000000097AA0101B55040B400B907004015B480B5C9E202",
                    INIT_0E => X"000006E32005FBC9D4020020B900000700C1B00150C6010000060400211BA00D",
                    INIT_0F => X"BE97C9DA0200260203E07200340205F57300300203EF65E002E96297BE003402",
                    INIT_10 => X"0F22300100000001D30140E0000034001FFF0001012D13000030010005020497",
                    INIT_11 => X"70809001552824540000C00100F6E401340000FE0601A0270234000000F61414",
                    INIT_12 => X"2CAB00401F013463055528246103610400C0D0B0A0BA2CAB3400401F0000FF06",
                    INIT_13 => X"02007F860000B00900008000017D0080750274CDD574000C806E019001D52CBA",
                    INIT_14 => X"43705400205F00940E019B8034000D060C070B52030A5208000901AB0000B000",
                    INIT_15 => X"035F200123240000AC00019BB800B402000001B9010001000174438043904360",
                    INIT_16 => X"2F2054534F50002F20544547CA0001010060400000A00000BF20005520001752",
                    INIT_17 => X"1001000003FF000101006369736142203A6E6F6974617A69726F68747541005F",
                    INIT_18 => X"00800007015508245F022D5F0010010C01F75F0C015F010152001D2DF7000000",
                    INIT_19 => X"BA0CAB1400021E00C00007015508245F03001D00005B550826BA0CAB14000240",
                    INIT_1A => X"437054607055082870600002054010005508262702160152001D5F2201000003",
                    INIT_1B => X"4300345F4001230C285040007060504001746B07051000706010000143184360",
                    INIT_1C => X"39B0261DB03631B0312CB02C27AB0E801D00317D00AB74430230430043604300",
                    INIT_1D => X"704280429042030074B1400130433034007431F431403A203BE036C037A03880",
                    INIT_1E => X"F13FD04270428042904202F4200074CB01D1FF30420042704280429042033F42",
                    INIT_1F => X"00000000000000003F043F063F420500ED01F13F704280429042D8F400F6DF01",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"0008C00282606B77692B520002040000000000005100000008444AAAA8AB1422",
                   INITP_01 => X"C9451559B25D4D43459DB6B9B13B5FD2AC73B2EED4600CB72154A503B0000844",
                   INITP_02 => X"857FFFFFFFF44C48BC40B100000808D38A8542A00A10A895C2062A8C05805145",
                   INITP_03 => X"0000A0168019E400007007FFDB6E9C000ECA25400986345811A2A888D1448922",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a_l(31 downto 0),
                  DOPA => data_out_a_l(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b_l(31 downto 0),
                  DOPB => data_out_b_l(35 downto 32), 
                   DIB => data_in_b_l(31 downto 0),
                  DIPB => data_in_b_l(35 downto 32), 
                   WEB => we_b(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
      -- 
      kcpsm6_rom_h: RAMB16BWER
      generic map ( DATA_WIDTH_A => 9,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 9,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"1AE8EAC8EA100090E8B8B0B801000101B068480B0C0C10105858B8005B005800",
                    INIT_01 => X"0028B0D9D8C800100808091008080910080809288A28CAC88AF0CAC8CAC88A28",
                    INIT_02 => X"2C0C000C2803000300001C10001C28010001000100095B2849B06949695B285B",
                    INIT_03 => X"0900090009000900090009000900090009000900090009000900095B285B0000",
                    INIT_04 => X"E959A8E9000C0C28B0E38B0051105B0009000900090009000900090009000900",
                    INIT_05 => X"0B0B0C0C005B000B0B000C5B005B5B0C005B0C0C5B000B0B0009000900095BB0",
                    INIT_06 => X"90E990E9000C0C10000B0C005B005910000B0C28B088E85880EB5B0C000B0C00",
                    INIT_07 => X"E990E990E990E990E90C91E991E9000C0C90E990E990E990E990E990E90C90E9",
                    INIT_08 => X"B8000B0C000C000C000C000C000C000C000C000C0B000C000C0B2891E991E990",
                    INIT_09 => X"EEB8B7B6B8000C0C28BFBFBEBEBDBD000B0C0C10000B0C0C100100002C0B0B0C",
                    INIT_0A => X"90FFEE9A8A8971000A0A090009000100015B000C0C050588FBEB90FBEBB9B0FF",
                    INIT_0B => X"B5B5B4B4B3B1B8A8B8EFA0EFB8000B0C000C0C93835B0404B1B0F2E290E990E9",
                    INIT_0C => X"B4B4B3B0B0280D0D0EA8EE0FB0FAFAE95A5A28098FA0EF80EF3588FDEDB8B7B6",
                    INIT_0D => X"0D0B280FB8B4B4B3B5B5B80E10F08A8BB0E15290E9010C0C0B0AB0EEB0010D0D",
                    INIT_0E => X"0B0C0CB0E959B0010D0D0E06010B0C0C28B0E58D23B0C8A4A4A308C80000520D",
                    INIT_0F => X"00B0010D0D280D0D0EB0E9280D0D0EB0E9280D0D0EB0E9100EB0E9D000280D0D",
                    INIT_10 => X"6F91E78FB89C9C8B01B8B1520F2891B8FFEE9F8EB800B1E9B8B1B8280D0D0ED0",
                    INIT_11 => X"B6B5B5B8000C0C91EEB8B6B8B80101B811B80B0C0CB8250D0D01280F0E011191",
                    INIT_12 => X"0C01B80A0AB80C91EE000C0C91EE91EE0EB6B3B4B4010C010CB80A0A0E0B0C0C",
                    INIT_13 => X"7B7C1C91EB7C010A097C2C0CB801B8B1B1EC110000B1E80C00B1EC01B8100C01",
                    INIT_14 => X"00015BB8B411B8B1EC8C0151BC0C7949794979000C79000C0C78B801B87C0109",
                    INIT_15 => X"0C000B0C000C28B8B1DACA0191E991EE9C9C8B01B828B89B8B5B000100010001",
                    INIT_16 => X"0A0A0A0A0A0A0A0A0A0A0A0A119D8D8AA8E25388EA250A28B1FBEB000CA8E900",
                    INIT_17 => X"B0B828B87FEE9F8EB80A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A",
                    INIT_18 => X"0A0B0C0CB8000C0C000B1000B8B3B80C0C01100C0CE0E90B000C0CB00188E8B8",
                    INIT_19 => X"110C010CB80E0A0A0B0C0CB8000C0C000B0C0CC8FBEB000C0C110C010CB80E0A",
                    INIT_1A => X"00015BB2B2000C0CB3B3A86949D1FBEB000C0C91E991E9000C0C100B0C88E949",
                    INIT_1B => X"00095B100B0C000C0C9383B8B3B3B2B2B85BB169496BB8B3B30B0BB800090001",
                    INIT_1C => X"59010A09010A09010A09010A09B1F9E95959000109115B002901000900090009",
                    INIT_1D => X"01000100010009285BB1E1897100515B285B005B006959695969596959695969",
                    INIT_1E => X"01000100010001000100090106285B118991E971000900010001000100091000",
                    INIT_1F => X"00000000000000001009100910000928B169010001000100010009012801B169",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"552A5552A99F94890E588CAB299755555555555F2ADAAB4FE1111555575F83DF",
                   INITP_01 => X"E451455C1308C41C282C942C0448256403AC844A831561224C8118B04AAAA535",
                   INITP_02 => X"70FFFFFFFFF895954B9A1E6AB722B67ED64EBC4D654A0540195FC66748C70EB1",
                   INITP_03 => X"00ADB55ED5575AAB55CDFD554924372AB210E61AA4249524A81912540CBA649D",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a_h(31 downto 0),
                  DOPA => data_out_a_h(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b_h(31 downto 0),
                  DOPB => data_out_b_h(35 downto 32), 
                   DIB => data_in_b_h(31 downto 0),
                  DIPB => data_in_b_h(35 downto 32), 
                   WEB => we_b(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
    --
    end generate s6;
    --
    --
    v6 : if (C_FAMILY = "V6") generate
      --
      address_a <= '1' & address(10 downto 0) & "1111";
      instruction <= data_out_a(33 downto 32) & data_out_a(15 downto 0);
      data_in_a <= "00000000000000000000000000000000000" & address(11);
      jtag_dout <= data_out_b(33 downto 32) & data_out_b(15 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b <= "00" & data_out_b(33 downto 32) & "0000000000000000" & data_out_b(15 downto 0);
        address_b <= "1111111111111111";
        we_b <= "00000000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b <= "00" & jtag_din(17 downto 16) & "0000000000000000" & jtag_din(15 downto 0);
        address_b <= '1' & jtag_addr(10 downto 0) & "1111";
        we_b <= jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      --
      kcpsm6_rom: RAMB36E1
      generic map ( READ_WIDTH_A => 18,
                    WRITE_WIDTH_A => 18,
                    DOA_REG => 0,
                    INIT_A => X"000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 18,
                    WRITE_WIDTH_B => 18,
                    DOB_REG => 0,
                    INIT_B => X"000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    EN_ECC_READ => FALSE,
                    EN_ECC_WRITE => FALSE,
                    RAM_EXTENSION_A => "NONE",
                    RAM_EXTENSION_B => "NONE",
                    SIM_DEVICE => "VIRTEX6",
                    INIT_00 => X"6011D04090001700180019072009200AB001B03171FF0035B6740031B1740035",
                    INIT_01 => X"35DFD000D57B9000D561201900D92009D1FF700061107001038B009903C203E4",
                    INIT_02 => X"1010112712005000150A500095F690001507E02B9511900095E9900015B95000",
                    INIT_03 => X"004250006039B200B10090010000203910801196129820391040110D12032039",
                    INIT_04 => X"004302800043029000431200B654500092046044D2019205D208B6645000B674",
                    INIT_05 => X"5804191F005F191E50000620006107200043004938FB2061004938FB50000270",
                    INIT_06 => X"12A8004312C000431280004312040043120000431200B6545000B67400430049",
                    INIT_07 => X"12080043120000431200004312FF004312FF004312FF00431201004312010043",
                    INIT_08 => X"120200431201004312A8004312C0004312030043120200431201004312DC0043",
                    INIT_09 => X"D280B2005000D20400521800193950006093C67016010043A2602106B6740043",
                    INIT_0A => X"005FB71318081900B674009317131600004312040043120000431200B6546064",
                    INIT_0B => X"16F41701191918000106B6740093171C16160049190CB6540123B615B7141904",
                    INIT_0C => X"002D17101901500061101000D1FFB11C005FD701B726191D005F1704191B0123",
                    INIT_0D => X"2128D20020EFD214005218281903205F002D170819010035B6740031B274205F",
                    INIT_0E => X"1903212DD21320CDD21D20CDD21B20CDD21A20CDD21820D5D21C182C2138D217",
                    INIT_0F => X"D21D20CDD21B20CDD21A20CDD21820D1D21C180C22FED2002346D21700521808",
                    INIT_10 => X"005C1868005C18481700005C1828005C18081708500022F7D215230ED21320CD",
                    INIT_11 => X"7100005F1701190000CD182800CD1808005C18E8005C18C8005C18A8005C1888",
                    INIT_12 => X"1702182C1901202D005F1701182C190120610260004300495804165017001904",
                    INIT_13 => X"DC0470006FF06CC0700100551828192650007F007E007D007C007B007A00005F",
                    INIT_14 => X"0270B6540055182819280B700A601000F700D6002179F700D60073006146FFFF",
                    INIT_15 => X"2162FFFFDC04350014011301E230004314001500130000431238004302600043",
                    INIT_16 => X"0123182C192827900680B6740950084063306158E5B0C4A02167D3FF2167D20A",
                    INIT_17 => X"6BB06AA06990688067706330700150007000DFFF4187DFFF7001005F17401901",
                    INIT_18 => X"B401B500500013001F01418EDF0001D0DF006BA01000FB00DA0070006FF06CC0",
                    INIT_19 => X"69906880677061B060A050001A2A1B021C005000DD801FFE619BF40AF50DD302",
                    INIT_1A => X"61B5C250A54021B4D20003B9190718001740141561B4DD8061B502C91AE21B02",
                    INIT_1B => X"1A07170050001E0070016990688067706B106A0070001D002197E1AA14011701",
                    INIT_1C => X"500061C1CAB01A01475061C69001490048004706100490000021001BA5A01B0D",
                    INIT_1D => X"17001800190661E3D220B20561FB02C91AD41B021C000D2003B9170018001907",
                    INIT_1E => X"D27350001A301B021C0361EFD26521E01C0261E9D262A19701BE50001A341B02",
                    INIT_1F => X"01BE619702C91ADA1B0250001A261B021C0361E0D27250001A341B021C0561F5",
                    INIT_20 => X"FE1FDDFF3E001D017001002D6213D30070006330700150001A051B021C04A197",
                    INIT_21 => X"DE0F2222CE301E01700039003800170103D370016240A4E01E00500022347000",
                    INIT_22 => X"22347000170018FE190670014BA01A271B02023450001E001D0003F622142214",
                    INIT_23 => X"6D706B806A9070010055182819242254DC0070006CC07001700003F603E47001",
                    INIT_24 => X"1D006CC067D068B069A002BA182C02AB193470001440151F1C00170018FF1906",
                    INIT_25 => X"182C02AB70001440151F700119342263DC050055182819242261DC032261DC04",
                    INIT_26 => X"6275D902227400CD00D56274D100180C0180626ED9010290700120D5182C02BA",
                    INIT_27 => X"F702F800387F2286D700F80003B014091300F800588018007001037D70006380",
                    INIT_28 => X"F20C9207F20B00521903F20A005218081900F109700103AB7000F80003B01300",
                    INIT_29 => X"00430270B65470006820225F70006294D80E1801029BA28079341800F20D9206",
                    INIT_2A => X"39003800170103B970015000700137001601B674004302800043029000430260",
                    INIT_2B => X"1903005F17201901012319245000700062ACB5009401029B22B8D20022B4DC02",
                    INIT_2C => X"5000C560A6401000D5004BA01400500062BFF720D600005519205000D2170052",
                    INIT_2D => X"152F152015541553154F15501500152F152015541545154722CA3B001A011401",
                    INIT_2E => X"153A156E156F156915741561157A15691572156F15681574157515411500155F",
                    INIT_2F => X"6110700150007000FE03DDFF3E001D0170011500156315691573156115421520",
                    INIT_30 => X"190102F7205F180C1901C05FD201170100521800191D602D02F71000D1007000",
                    INIT_31 => X"15001780180019077001005518081924005F1702202D005F700067107001180C",
                    INIT_32 => X"17031800191D9000F700D65B00551808192622BA180C02AB191470001C021440",
                    INIT_33 => X"22BA180C02AB191470001C02141E150017C0180019077001005518081924005F",
                    INIT_34 => X"0055180819262327D2022316D20100521800191D205F172219011000D2009203",
                    INIT_35 => X"00430270B65464606570005518081928677066605000D2029205A340F710D600",
                    INIT_36 => X"7001B674636BD2079205D7107000677066601710160070010043121800430260",
                    INIT_37 => X"00431200B634205F174019010123180C19282750064070006770666065506440",
                    INIT_38 => X"B31DB2000031037D130023ABB674004352020230004312000043126000431200",
                    INIT_39 => X"B33903B01426131D03B01436133103B01431132C03B0142C132763ABF30ED280",
                    INIT_3A => X"5000B6740031B6F40031D340B33AD320B33BD3E0B336D3C0B337D3A0B338D380",
                    INIT_3B => X"02700042028000420290004212035000B67463B1C3401301E2300043A230B634",
                    INIT_3C => X"130123D1D3FFE2300042130000420270004202800042029000421203203F0042",
                    INIT_3D => X"03F1003F02D00042027000420280004202900042120203F40D205000B67423CB",
                    INIT_3E => X"63EDD20103F1003F02700042028000420290004212D803F4500003F663DFD201",
                    INIT_3F => X"00000000000000000000000000000000203F1204203F1206203F004212055000",
                    INIT_40 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_41 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_42 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_43 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_44 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_45 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_46 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_47 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_48 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_49 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_50 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_51 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_52 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_53 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_54 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_55 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_56 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_57 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_58 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_59 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_60 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_61 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_62 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_63 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_64 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_65 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_66 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_67 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_68 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_69 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_70 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_71 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_72 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_73 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_74 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_75 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_76 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_77 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_78 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_79 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"22222222222222AA0888A288888A30AAAD52020202677776376B77AAC00AA6AA",
                   INITP_01 => X"77774DD83777774DDD8282AA8274D08200A8228080A0888B4D82D62A22222222",
                   INITP_02 => X"555FDDE0816075DDD55A022228035D5D75E0955602808A006088888888220B77",
                   INITP_03 => X"B8203603603637A0034E0080B53553A008D55CB5D3600DE0556034D508776D75",
                   INITP_04 => X"A36820EDDAB434E22C3360DD15623000578377EBB038282B35D5B42F55EDDE03",
                   INITP_05 => X"7B15EAAAAAAAAAAAAAAAAA95D362D636208BD6DD56ED68888B6F58488A282EE8",
                   INITP_06 => X"8A081755EC2D438889605C358376083488C003820358223000E08ADC283483B7",
                   INITP_07 => X"000088A2CA2222ACA222222A7688888A2222B5A2AAA22222208208350A2A0888",
                   INITP_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(   ADDRARDADDR => address_a,
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a(31 downto 0),
                      DOPADOP => data_out_a(35 downto 32), 
                        DIADI => data_in_a(31 downto 0),
                      DIPADIP => data_in_a(35 downto 32), 
                          WEA => "0000",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b,
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b(31 downto 0),
                      DOPBDOP => data_out_b(35 downto 32), 
                        DIBDI => data_in_b(31 downto 0),
                      DIPBDIP => data_in_b(35 downto 32), 
                        WEBWE => we_b,
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0',
                   CASCADEINA => '0',
                   CASCADEINB => '0',
                INJECTDBITERR => '0',
                INJECTSBITERR => '0');
      --
    end generate v6;
    --
    --
    akv7 : if (C_FAMILY = "7S") generate
      --
      address_a <= '1' & address(10 downto 0) & "1111";
      instruction <= data_out_a(33 downto 32) & data_out_a(15 downto 0);
      data_in_a <= "00000000000000000000000000000000000" & address(11);
      jtag_dout <= data_out_b(33 downto 32) & data_out_b(15 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b <= "00" & data_out_b(33 downto 32) & "0000000000000000" & data_out_b(15 downto 0);
        address_b <= "1111111111111111";
        we_b <= "00000000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b <= "00" & jtag_din(17 downto 16) & "0000000000000000" & jtag_din(15 downto 0);
        address_b <= '1' & jtag_addr(10 downto 0) & "1111";
        we_b <= jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      --
      kcpsm6_rom: RAMB36E1
      generic map ( READ_WIDTH_A => 18,
                    WRITE_WIDTH_A => 18,
                    DOA_REG => 0,
                    INIT_A => X"000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 18,
                    WRITE_WIDTH_B => 18,
                    DOB_REG => 0,
                    INIT_B => X"000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    EN_ECC_READ => FALSE,
                    EN_ECC_WRITE => FALSE,
                    RAM_EXTENSION_A => "NONE",
                    RAM_EXTENSION_B => "NONE",
                    SIM_DEVICE => "7SERIES",
                    INIT_00 => X"6011D04090001700180019072009200AB001B03171FF0035B6740031B1740035",
                    INIT_01 => X"35DFD000D57B9000D561201900D92009D1FF700061107001038B009903C203E4",
                    INIT_02 => X"1010112712005000150A500095F690001507E02B9511900095E9900015B95000",
                    INIT_03 => X"004250006039B200B10090010000203910801196129820391040110D12032039",
                    INIT_04 => X"004302800043029000431200B654500092046044D2019205D208B6645000B674",
                    INIT_05 => X"5804191F005F191E50000620006107200043004938FB2061004938FB50000270",
                    INIT_06 => X"12A8004312C000431280004312040043120000431200B6545000B67400430049",
                    INIT_07 => X"12080043120000431200004312FF004312FF004312FF00431201004312010043",
                    INIT_08 => X"120200431201004312A8004312C0004312030043120200431201004312DC0043",
                    INIT_09 => X"D280B2005000D20400521800193950006093C67016010043A2602106B6740043",
                    INIT_0A => X"005FB71318081900B674009317131600004312040043120000431200B6546064",
                    INIT_0B => X"16F41701191918000106B6740093171C16160049190CB6540123B615B7141904",
                    INIT_0C => X"002D17101901500061101000D1FFB11C005FD701B726191D005F1704191B0123",
                    INIT_0D => X"2128D20020EFD214005218281903205F002D170819010035B6740031B274205F",
                    INIT_0E => X"1903212DD21320CDD21D20CDD21B20CDD21A20CDD21820D5D21C182C2138D217",
                    INIT_0F => X"D21D20CDD21B20CDD21A20CDD21820D1D21C180C22FED2002346D21700521808",
                    INIT_10 => X"005C1868005C18481700005C1828005C18081708500022F7D215230ED21320CD",
                    INIT_11 => X"7100005F1701190000CD182800CD1808005C18E8005C18C8005C18A8005C1888",
                    INIT_12 => X"1702182C1901202D005F1701182C190120610260004300495804165017001904",
                    INIT_13 => X"DC0470006FF06CC0700100551828192650007F007E007D007C007B007A00005F",
                    INIT_14 => X"0270B6540055182819280B700A601000F700D6002179F700D60073006146FFFF",
                    INIT_15 => X"2162FFFFDC04350014011301E230004314001500130000431238004302600043",
                    INIT_16 => X"0123182C192827900680B6740950084063306158E5B0C4A02167D3FF2167D20A",
                    INIT_17 => X"6BB06AA06990688067706330700150007000DFFF4187DFFF7001005F17401901",
                    INIT_18 => X"B401B500500013001F01418EDF0001D0DF006BA01000FB00DA0070006FF06CC0",
                    INIT_19 => X"69906880677061B060A050001A2A1B021C005000DD801FFE619BF40AF50DD302",
                    INIT_1A => X"61B5C250A54021B4D20003B9190718001740141561B4DD8061B502C91AE21B02",
                    INIT_1B => X"1A07170050001E0070016990688067706B106A0070001D002197E1AA14011701",
                    INIT_1C => X"500061C1CAB01A01475061C69001490048004706100490000021001BA5A01B0D",
                    INIT_1D => X"17001800190661E3D220B20561FB02C91AD41B021C000D2003B9170018001907",
                    INIT_1E => X"D27350001A301B021C0361EFD26521E01C0261E9D262A19701BE50001A341B02",
                    INIT_1F => X"01BE619702C91ADA1B0250001A261B021C0361E0D27250001A341B021C0561F5",
                    INIT_20 => X"FE1FDDFF3E001D017001002D6213D30070006330700150001A051B021C04A197",
                    INIT_21 => X"DE0F2222CE301E01700039003800170103D370016240A4E01E00500022347000",
                    INIT_22 => X"22347000170018FE190670014BA01A271B02023450001E001D0003F622142214",
                    INIT_23 => X"6D706B806A9070010055182819242254DC0070006CC07001700003F603E47001",
                    INIT_24 => X"1D006CC067D068B069A002BA182C02AB193470001440151F1C00170018FF1906",
                    INIT_25 => X"182C02AB70001440151F700119342263DC050055182819242261DC032261DC04",
                    INIT_26 => X"6275D902227400CD00D56274D100180C0180626ED9010290700120D5182C02BA",
                    INIT_27 => X"F702F800387F2286D700F80003B014091300F800588018007001037D70006380",
                    INIT_28 => X"F20C9207F20B00521903F20A005218081900F109700103AB7000F80003B01300",
                    INIT_29 => X"00430270B65470006820225F70006294D80E1801029BA28079341800F20D9206",
                    INIT_2A => X"39003800170103B970015000700137001601B674004302800043029000430260",
                    INIT_2B => X"1903005F17201901012319245000700062ACB5009401029B22B8D20022B4DC02",
                    INIT_2C => X"5000C560A6401000D5004BA01400500062BFF720D600005519205000D2170052",
                    INIT_2D => X"152F152015541553154F15501500152F152015541545154722CA3B001A011401",
                    INIT_2E => X"153A156E156F156915741561157A15691572156F15681574157515411500155F",
                    INIT_2F => X"6110700150007000FE03DDFF3E001D0170011500156315691573156115421520",
                    INIT_30 => X"190102F7205F180C1901C05FD201170100521800191D602D02F71000D1007000",
                    INIT_31 => X"15001780180019077001005518081924005F1702202D005F700067107001180C",
                    INIT_32 => X"17031800191D9000F700D65B00551808192622BA180C02AB191470001C021440",
                    INIT_33 => X"22BA180C02AB191470001C02141E150017C0180019077001005518081924005F",
                    INIT_34 => X"0055180819262327D2022316D20100521800191D205F172219011000D2009203",
                    INIT_35 => X"00430270B65464606570005518081928677066605000D2029205A340F710D600",
                    INIT_36 => X"7001B674636BD2079205D7107000677066601710160070010043121800430260",
                    INIT_37 => X"00431200B634205F174019010123180C19282750064070006770666065506440",
                    INIT_38 => X"B31DB2000031037D130023ABB674004352020230004312000043126000431200",
                    INIT_39 => X"B33903B01426131D03B01436133103B01431132C03B0142C132763ABF30ED280",
                    INIT_3A => X"5000B6740031B6F40031D340B33AD320B33BD3E0B336D3C0B337D3A0B338D380",
                    INIT_3B => X"02700042028000420290004212035000B67463B1C3401301E2300043A230B634",
                    INIT_3C => X"130123D1D3FFE2300042130000420270004202800042029000421203203F0042",
                    INIT_3D => X"03F1003F02D00042027000420280004202900042120203F40D205000B67423CB",
                    INIT_3E => X"63EDD20103F1003F02700042028000420290004212D803F4500003F663DFD201",
                    INIT_3F => X"00000000000000000000000000000000203F1204203F1206203F004212055000",
                    INIT_40 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_41 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_42 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_43 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_44 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_45 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_46 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_47 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_48 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_49 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_50 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_51 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_52 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_53 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_54 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_55 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_56 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_57 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_58 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_59 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_60 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_61 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_62 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_63 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_64 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_65 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_66 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_67 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_68 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_69 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_70 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_71 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_72 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_73 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_74 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_75 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_76 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_77 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_78 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_79 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"22222222222222AA0888A288888A30AAAD52020202677776376B77AAC00AA6AA",
                   INITP_01 => X"77774DD83777774DDD8282AA8274D08200A8228080A0888B4D82D62A22222222",
                   INITP_02 => X"555FDDE0816075DDD55A022228035D5D75E0955602808A006088888888220B77",
                   INITP_03 => X"B8203603603637A0034E0080B53553A008D55CB5D3600DE0556034D508776D75",
                   INITP_04 => X"A36820EDDAB434E22C3360DD15623000578377EBB038282B35D5B42F55EDDE03",
                   INITP_05 => X"7B15EAAAAAAAAAAAAAAAAA95D362D636208BD6DD56ED68888B6F58488A282EE8",
                   INITP_06 => X"8A081755EC2D438889605C358376083488C003820358223000E08ADC283483B7",
                   INITP_07 => X"000088A2CA2222ACA222222A7688888A2222B5A2AAA22222208208350A2A0888",
                   INITP_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(   ADDRARDADDR => address_a,
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a(31 downto 0),
                      DOPADOP => data_out_a(35 downto 32), 
                        DIADI => data_in_a(31 downto 0),
                      DIPADIP => data_in_a(35 downto 32), 
                          WEA => "0000",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b,
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b(31 downto 0),
                      DOPBDOP => data_out_b(35 downto 32), 
                        DIBDI => data_in_b(31 downto 0),
                      DIPBDIP => data_in_b(35 downto 32), 
                        WEBWE => we_b,
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0',
                   CASCADEINA => '0',
                   CASCADEINB => '0',
                INJECTDBITERR => '0',
                INJECTSBITERR => '0');
      --
    end generate akv7;
    --
  end generate ram_2k_generate;
  --
  --	
  ram_4k_generate : if (C_RAM_SIZE_KWORDS = 4) generate
    s6: if (C_FAMILY = "S6") generate
      --
      address_a(13 downto 0) <= address(10 downto 0) & "000";
      data_in_a <= "000000000000000000000000000000000000";
      --
      s6_a11_flop: FD
      port map (  D => address(11),
                  Q => pipe_a11,
                  C => clk);
      --
      s6_4k_mux0_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_ll(0),
                I1 => data_out_a_hl(0),
                I2 => data_out_a_ll(1),
                I3 => data_out_a_hl(1),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(0),
                O6 => instruction(1));
      --
      s6_4k_mux2_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_ll(2),
                I1 => data_out_a_hl(2),
                I2 => data_out_a_ll(3),
                I3 => data_out_a_hl(3),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(2),
                O6 => instruction(3));
      --
      s6_4k_mux4_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_ll(4),
                I1 => data_out_a_hl(4),
                I2 => data_out_a_ll(5),
                I3 => data_out_a_hl(5),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(4),
                O6 => instruction(5));
      --
      s6_4k_mux6_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_ll(6),
                I1 => data_out_a_hl(6),
                I2 => data_out_a_ll(7),
                I3 => data_out_a_hl(7),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(6),
                O6 => instruction(7));
      --
      s6_4k_mux8_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_ll(32),
                I1 => data_out_a_hl(32),
                I2 => data_out_a_lh(0),
                I3 => data_out_a_hh(0),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(8),
                O6 => instruction(9));
      --
      s6_4k_mux10_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_lh(1),
                I1 => data_out_a_hh(1),
                I2 => data_out_a_lh(2),
                I3 => data_out_a_hh(2),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(10),
                O6 => instruction(11));
      --
      s6_4k_mux12_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_lh(3),
                I1 => data_out_a_hh(3),
                I2 => data_out_a_lh(4),
                I3 => data_out_a_hh(4),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(12),
                O6 => instruction(13));
      --
      s6_4k_mux14_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_lh(5),
                I1 => data_out_a_hh(5),
                I2 => data_out_a_lh(6),
                I3 => data_out_a_hh(6),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(14),
                O6 => instruction(15));
      --
      s6_4k_mux16_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_lh(7),
                I1 => data_out_a_hh(7),
                I2 => data_out_a_lh(32),
                I3 => data_out_a_hh(32),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(16),
                O6 => instruction(17));
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b_ll <= "000" & data_out_b_ll(32) & "000000000000000000000000" & data_out_b_ll(7 downto 0);
        data_in_b_lh <= "000" & data_out_b_lh(32) & "000000000000000000000000" & data_out_b_lh(7 downto 0);
        data_in_b_hl <= "000" & data_out_b_hl(32) & "000000000000000000000000" & data_out_b_hl(7 downto 0);
        data_in_b_hh <= "000" & data_out_b_hh(32) & "000000000000000000000000" & data_out_b_hh(7 downto 0);
        address_b(13 downto 0) <= "00000000000000";
        we_b_l(3 downto 0) <= "0000";
        we_b_h(3 downto 0) <= "0000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
        jtag_dout <= data_out_b_lh(32) & data_out_b_lh(7 downto 0) & data_out_b_ll(32) & data_out_b_ll(7 downto 0);
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b_lh <= "000" & jtag_din(17) & "000000000000000000000000" & jtag_din(16 downto 9);
        data_in_b_ll <= "000" & jtag_din(8) & "000000000000000000000000" & jtag_din(7 downto 0);
        data_in_b_hh <= "000" & jtag_din(17) & "000000000000000000000000" & jtag_din(16 downto 9);
        data_in_b_hl <= "000" & jtag_din(8) & "000000000000000000000000" & jtag_din(7 downto 0);
        address_b(13 downto 0) <= jtag_addr(10 downto 0) & "000";
        --
        s6_4k_jtag_we_lut: LUT6_2
        generic map (INIT => X"8000000020000000")
        port map( I0 => jtag_we,
                  I1 => jtag_addr(11),
                  I2 => '1',
                  I3 => '1',
                  I4 => '1',
                  I5 => '1',
                  O5 => jtag_we_l,
                  O6 => jtag_we_h);
        --
        we_b_l(3 downto 0) <= jtag_we_l & jtag_we_l & jtag_we_l & jtag_we_l;
        we_b_h(3 downto 0) <= jtag_we_h & jtag_we_h & jtag_we_h & jtag_we_h;
        --
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
        --
        s6_4k_jtag_mux0_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_ll(0),
                  I1 => data_out_b_hl(0),
                  I2 => data_out_b_ll(1),
                  I3 => data_out_b_hl(1),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(0),
                  O6 => jtag_dout(1));
        --
        s6_4k_jtag_mux2_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_ll(2),
                  I1 => data_out_b_hl(2),
                  I2 => data_out_b_ll(3),
                  I3 => data_out_b_hl(3),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(2),
                  O6 => jtag_dout(3));
        --
        s6_4k_jtag_mux4_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_ll(4),
                  I1 => data_out_b_hl(4),
                  I2 => data_out_b_ll(5),
                  I3 => data_out_b_hl(5),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(4),
                  O6 => jtag_dout(5));
        --
        s6_4k_jtag_mux6_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_ll(6),
                  I1 => data_out_b_hl(6),
                  I2 => data_out_b_ll(7),
                  I3 => data_out_b_hl(7),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(6),
                  O6 => jtag_dout(7));
        --
        s6_4k_jtag_mux8_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_ll(32),
                  I1 => data_out_b_hl(32),
                  I2 => data_out_b_lh(0),
                  I3 => data_out_b_hh(0),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(8),
                  O6 => jtag_dout(9));
        --
        s6_4k_jtag_mux10_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_lh(1),
                  I1 => data_out_b_hh(1),
                  I2 => data_out_b_lh(2),
                  I3 => data_out_b_hh(2),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(10),
                  O6 => jtag_dout(11));
        --
        s6_4k_jtag_mux12_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_lh(3),
                  I1 => data_out_b_hh(3),
                  I2 => data_out_b_lh(4),
                  I3 => data_out_b_hh(4),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(12),
                  O6 => jtag_dout(13));
        --
        s6_4k_jtag_mux14_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_lh(5),
                  I1 => data_out_b_hh(5),
                  I2 => data_out_b_lh(6),
                  I3 => data_out_b_hh(6),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(14),
                  O6 => jtag_dout(15));
        --
        s6_4k_jtag_mux16_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_lh(7),
                  I1 => data_out_b_hh(7),
                  I2 => data_out_b_lh(32),
                  I3 => data_out_b_hh(32),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(16),
                  O6 => jtag_dout(17));
      --
      end generate loader;
      --
      kcpsm6_rom_ll: RAMB16BWER
      generic map ( DATA_WIDTH_A => 9,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 9,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"DF007B006119D909FF0010018B99C2E4114000000007090A0131FF3574317435",
                    INIT_01 => X"420039000001003980969839400D0339102700000A00F600072B1100E900B900",
                    INIT_02 => X"041F5F1E002061204349FB6149FB007043804390430054000444010508640074",
                    INIT_03 => X"084300430043FF43FF43FF4301430143A843C043804304430043005400744349",
                    INIT_04 => X"8000000452003900937001436006744302430143A843C043034302430143DC43",
                    INIT_05 => X"F40119000674931C16490C54231514045F130800749313004304430043005464",
                    INIT_06 => X"2800EF145228035F2D0801357431745F2D1001001000FF1C5F01261D5F041B23",
                    INIT_07 => X"1DCD1BCD1ACD18D11C0CFE0046175208032D13CD1DCD1BCD1ACD18D51C2C3817",
                    INIT_08 => X"005F0100CD28CD085CE85CC85CA85C885C685C48005C285C080800F7150E13CD",
                    INIT_09 => X"0400F0C001552826000000000000005F022C012D5F012C016160434904500004",
                    INIT_0A => X"62FF0400010130430000004338436043705455282870600000007900000046FF",
                    INIT_0B => X"B0A090807030010000FF87FF015F4001232C2890807450403058B0A067FF670A",
                    INIT_0C => X"908070B0A0002A02000080FE9B0A0D0201000000018E00D000A000000000F0C0",
                    INIT_0D => X"07000000019080701000000097AA0101B55040B400B907004015B480B5C9E202",
                    INIT_0E => X"000006E32005FBC9D4020020B900000700C1B00150C6010000060400211BA00D",
                    INIT_0F => X"BE97C9DA0200260203E07200340205F57300300203EF65E002E96297BE003402",
                    INIT_10 => X"0F22300100000001D30140E0000034001FFF0001012D13000030010005020497",
                    INIT_11 => X"70809001552824540000C00100F6E401340000FE0601A0270234000000F61414",
                    INIT_12 => X"2CAB00401F013463055528246103610400C0D0B0A0BA2CAB3400401F0000FF06",
                    INIT_13 => X"02007F860000B00900008000017D0080750274CDD574000C806E019001D52CBA",
                    INIT_14 => X"43705400205F00940E019B8034000D060C070B52030A5208000901AB0000B000",
                    INIT_15 => X"035F200123240000AC00019BB800B402000001B9010001000174438043904360",
                    INIT_16 => X"2F2054534F50002F20544547CA0001010060400000A00000BF20005520001752",
                    INIT_17 => X"1001000003FF000101006369736142203A6E6F6974617A69726F68747541005F",
                    INIT_18 => X"00800007015508245F022D5F0010010C01F75F0C015F010152001D2DF7000000",
                    INIT_19 => X"BA0CAB1400021E00C00007015508245F03001D00005B550826BA0CAB14000240",
                    INIT_1A => X"437054607055082870600002054010005508262702160152001D5F2201000003",
                    INIT_1B => X"4300345F4001230C285040007060504001746B07051000706010000143184360",
                    INIT_1C => X"39B0261DB03631B0312CB02C27AB0E801D00317D00AB74430230430043604300",
                    INIT_1D => X"704280429042030074B1400130433034007431F431403A203BE036C037A03880",
                    INIT_1E => X"F13FD04270428042904202F4200074CB01D1FF30420042704280429042033F42",
                    INIT_1F => X"00000000000000003F043F063F420500ED01F13F704280429042D8F400F6DF01",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"0008C00282606B77692B520002040000000000005100000008444AAAA8AB1422",
                   INITP_01 => X"C9451559B25D4D43459DB6B9B13B5FD2AC73B2EED4600CB72154A503B0000844",
                   INITP_02 => X"857FFFFFFFF44C48BC40B100000808D38A8542A00A10A895C2062A8C05805145",
                   INITP_03 => X"0000A0168019E400007007FFDB6E9C000ECA25400986345811A2A888D1448922",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a_ll(31 downto 0),
                  DOPA => data_out_a_ll(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b_ll(31 downto 0),
                  DOPB => data_out_b_ll(35 downto 32), 
                   DIB => data_in_b_ll(31 downto 0),
                  DIPB => data_in_b_ll(35 downto 32), 
                   WEB => we_b_l(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
      -- 
      kcpsm6_rom_lh: RAMB16BWER
      generic map ( DATA_WIDTH_A => 9,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 9,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"1AE8EAC8EA100090E8B8B0B801000101B068480B0C0C10105858B8005B005800",
                    INIT_01 => X"0028B0D9D8C800100808091008080910080809288A28CAC88AF0CAC8CAC88A28",
                    INIT_02 => X"2C0C000C2803000300001C10001C28010001000100095B2849B06949695B285B",
                    INIT_03 => X"0900090009000900090009000900090009000900090009000900095B285B0000",
                    INIT_04 => X"E959A8E9000C0C28B0E38B0051105B0009000900090009000900090009000900",
                    INIT_05 => X"0B0B0C0C005B000B0B000C5B005B5B0C005B0C0C5B000B0B0009000900095BB0",
                    INIT_06 => X"90E990E9000C0C10000B0C005B005910000B0C28B088E85880EB5B0C000B0C00",
                    INIT_07 => X"E990E990E990E990E90C91E991E9000C0C90E990E990E990E990E990E90C90E9",
                    INIT_08 => X"B8000B0C000C000C000C000C000C000C000C000C0B000C000C0B2891E991E990",
                    INIT_09 => X"EEB8B7B6B8000C0C28BFBFBEBEBDBD000B0C0C10000B0C0C100100002C0B0B0C",
                    INIT_0A => X"90FFEE9A8A8971000A0A090009000100015B000C0C050588FBEB90FBEBB9B0FF",
                    INIT_0B => X"B5B5B4B4B3B1B8A8B8EFA0EFB8000B0C000C0C93835B0404B1B0F2E290E990E9",
                    INIT_0C => X"B4B4B3B0B0280D0D0EA8EE0FB0FAFAE95A5A28098FA0EF80EF3588FDEDB8B7B6",
                    INIT_0D => X"0D0B280FB8B4B4B3B5B5B80E10F08A8BB0E15290E9010C0C0B0AB0EEB0010D0D",
                    INIT_0E => X"0B0C0CB0E959B0010D0D0E06010B0C0C28B0E58D23B0C8A4A4A308C80000520D",
                    INIT_0F => X"00B0010D0D280D0D0EB0E9280D0D0EB0E9280D0D0EB0E9100EB0E9D000280D0D",
                    INIT_10 => X"6F91E78FB89C9C8B01B8B1520F2891B8FFEE9F8EB800B1E9B8B1B8280D0D0ED0",
                    INIT_11 => X"B6B5B5B8000C0C91EEB8B6B8B80101B811B80B0C0CB8250D0D01280F0E011191",
                    INIT_12 => X"0C01B80A0AB80C91EE000C0C91EE91EE0EB6B3B4B4010C010CB80A0A0E0B0C0C",
                    INIT_13 => X"7B7C1C91EB7C010A097C2C0CB801B8B1B1EC110000B1E80C00B1EC01B8100C01",
                    INIT_14 => X"00015BB8B411B8B1EC8C0151BC0C7949794979000C79000C0C78B801B87C0109",
                    INIT_15 => X"0C000B0C000C28B8B1DACA0191E991EE9C9C8B01B828B89B8B5B000100010001",
                    INIT_16 => X"0A0A0A0A0A0A0A0A0A0A0A0A119D8D8AA8E25388EA250A28B1FBEB000CA8E900",
                    INIT_17 => X"B0B828B87FEE9F8EB80A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A",
                    INIT_18 => X"0A0B0C0CB8000C0C000B1000B8B3B80C0C01100C0CE0E90B000C0CB00188E8B8",
                    INIT_19 => X"110C010CB80E0A0A0B0C0CB8000C0C000B0C0CC8FBEB000C0C110C010CB80E0A",
                    INIT_1A => X"00015BB2B2000C0CB3B3A86949D1FBEB000C0C91E991E9000C0C100B0C88E949",
                    INIT_1B => X"00095B100B0C000C0C9383B8B3B3B2B2B85BB169496BB8B3B30B0BB800090001",
                    INIT_1C => X"59010A09010A09010A09010A09B1F9E95959000109115B002901000900090009",
                    INIT_1D => X"01000100010009285BB1E1897100515B285B005B006959695969596959695969",
                    INIT_1E => X"01000100010001000100090106285B118991E971000900010001000100091000",
                    INIT_1F => X"00000000000000001009100910000928B169010001000100010009012801B169",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"552A5552A99F94890E588CAB299755555555555F2ADAAB4FE1111555575F83DF",
                   INITP_01 => X"E451455C1308C41C282C942C0448256403AC844A831561224C8118B04AAAA535",
                   INITP_02 => X"70FFFFFFFFF895954B9A1E6AB722B67ED64EBC4D654A0540195FC66748C70EB1",
                   INITP_03 => X"00ADB55ED5575AAB55CDFD554924372AB210E61AA4249524A81912540CBA649D",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a_lh(31 downto 0),
                  DOPA => data_out_a_lh(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b_lh(31 downto 0),
                  DOPB => data_out_b_lh(35 downto 32), 
                   DIB => data_in_b_lh(31 downto 0),
                  DIPB => data_in_b_lh(35 downto 32), 
                   WEB => we_b_l(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
      --
      kcpsm6_rom_hl: RAMB16BWER
      generic map ( DATA_WIDTH_A => 9,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 9,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_01 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_02 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_03 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_10 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_11 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_12 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_13 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_14 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_15 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_16 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_17 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_18 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_19 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_01 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_02 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_03 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a_hl(31 downto 0),
                  DOPA => data_out_a_hl(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b_hl(31 downto 0),
                  DOPB => data_out_b_hl(35 downto 32), 
                   DIB => data_in_b_hl(31 downto 0),
                  DIPB => data_in_b_hl(35 downto 32), 
                   WEB => we_b_h(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
      -- 
      kcpsm6_rom_hh: RAMB16BWER
      generic map ( DATA_WIDTH_A => 9,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 9,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_01 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_02 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_03 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_10 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_11 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_12 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_13 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_14 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_15 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_16 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_17 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_18 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_19 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_01 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_02 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_03 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a_hh(31 downto 0),
                  DOPA => data_out_a_hh(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b_hh(31 downto 0),
                  DOPB => data_out_b_hh(35 downto 32), 
                   DIB => data_in_b_hh(31 downto 0),
                  DIPB => data_in_b_hh(35 downto 32), 
                   WEB => we_b_h(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
    --
    end generate s6;
    --
    --
    v6 : if (C_FAMILY = "V6") generate
      --
      address_a <= '1' & address(11 downto 0) & "111";
      instruction <= data_out_a_h(32) & data_out_a_h(7 downto 0) & data_out_a_l(32) & data_out_a_l(7 downto 0);
      data_in_a <= "000000000000000000000000000000000000";
      jtag_dout <= data_out_b_h(32) & data_out_b_h(7 downto 0) & data_out_b_l(32) & data_out_b_l(7 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b_l <= "000" & data_out_b_l(32) & "000000000000000000000000" & data_out_b_l(7 downto 0);
        data_in_b_h <= "000" & data_out_b_h(32) & "000000000000000000000000" & data_out_b_h(7 downto 0);
        address_b <= "1111111111111111";
        we_b <= "00000000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b_h <= "000" & jtag_din(17) & "000000000000000000000000" & jtag_din(16 downto 9);
        data_in_b_l <= "000" & jtag_din(8) & "000000000000000000000000" & jtag_din(7 downto 0);
        address_b <= '1' & jtag_addr(11 downto 0) & "111";
        we_b <= jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      --
      kcpsm6_rom_l: RAMB36E1
      generic map ( READ_WIDTH_A => 9,
                    WRITE_WIDTH_A => 9,
                    DOA_REG => 0,
                    INIT_A => X"000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 9,
                    WRITE_WIDTH_B => 9,
                    DOB_REG => 0,
                    INIT_B => X"000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    EN_ECC_READ => FALSE,
                    EN_ECC_WRITE => FALSE,
                    RAM_EXTENSION_A => "NONE",
                    RAM_EXTENSION_B => "NONE",
                    SIM_DEVICE => "VIRTEX6",
                    INIT_00 => X"DF007B006119D909FF0010018B99C2E4114000000007090A0131FF3574317435",
                    INIT_01 => X"420039000001003980969839400D0339102700000A00F600072B1100E900B900",
                    INIT_02 => X"041F5F1E002061204349FB6149FB007043804390430054000444010508640074",
                    INIT_03 => X"084300430043FF43FF43FF4301430143A843C043804304430043005400744349",
                    INIT_04 => X"8000000452003900937001436006744302430143A843C043034302430143DC43",
                    INIT_05 => X"F40119000674931C16490C54231514045F130800749313004304430043005464",
                    INIT_06 => X"2800EF145228035F2D0801357431745F2D1001001000FF1C5F01261D5F041B23",
                    INIT_07 => X"1DCD1BCD1ACD18D11C0CFE0046175208032D13CD1DCD1BCD1ACD18D51C2C3817",
                    INIT_08 => X"005F0100CD28CD085CE85CC85CA85C885C685C48005C285C080800F7150E13CD",
                    INIT_09 => X"0400F0C001552826000000000000005F022C012D5F012C016160434904500004",
                    INIT_0A => X"62FF0400010130430000004338436043705455282870600000007900000046FF",
                    INIT_0B => X"B0A090807030010000FF87FF015F4001232C2890807450403058B0A067FF670A",
                    INIT_0C => X"908070B0A0002A02000080FE9B0A0D0201000000018E00D000A000000000F0C0",
                    INIT_0D => X"07000000019080701000000097AA0101B55040B400B907004015B480B5C9E202",
                    INIT_0E => X"000006E32005FBC9D4020020B900000700C1B00150C6010000060400211BA00D",
                    INIT_0F => X"BE97C9DA0200260203E07200340205F57300300203EF65E002E96297BE003402",
                    INIT_10 => X"0F22300100000001D30140E0000034001FFF0001012D13000030010005020497",
                    INIT_11 => X"70809001552824540000C00100F6E401340000FE0601A0270234000000F61414",
                    INIT_12 => X"2CAB00401F013463055528246103610400C0D0B0A0BA2CAB3400401F0000FF06",
                    INIT_13 => X"02007F860000B00900008000017D0080750274CDD574000C806E019001D52CBA",
                    INIT_14 => X"43705400205F00940E019B8034000D060C070B52030A5208000901AB0000B000",
                    INIT_15 => X"035F200123240000AC00019BB800B402000001B9010001000174438043904360",
                    INIT_16 => X"2F2054534F50002F20544547CA0001010060400000A00000BF20005520001752",
                    INIT_17 => X"1001000003FF000101006369736142203A6E6F6974617A69726F68747541005F",
                    INIT_18 => X"00800007015508245F022D5F0010010C01F75F0C015F010152001D2DF7000000",
                    INIT_19 => X"BA0CAB1400021E00C00007015508245F03001D00005B550826BA0CAB14000240",
                    INIT_1A => X"437054607055082870600002054010005508262702160152001D5F2201000003",
                    INIT_1B => X"4300345F4001230C285040007060504001746B07051000706010000143184360",
                    INIT_1C => X"39B0261DB03631B0312CB02C27AB0E801D00317D00AB74430230430043604300",
                    INIT_1D => X"704280429042030074B1400130433034007431F431403A203BE036C037A03880",
                    INIT_1E => X"F13FD04270428042904202F4200074CB01D1FF30420042704280429042033F42",
                    INIT_1F => X"00000000000000003F043F063F420500ED01F13F704280429042D8F400F6DF01",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_40 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_41 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_42 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_43 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_44 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_45 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_46 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_47 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_48 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_49 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_50 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_51 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_52 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_53 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_54 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_55 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_56 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_57 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_58 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_59 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_60 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_61 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_62 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_63 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_64 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_65 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_66 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_67 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_68 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_69 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_70 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_71 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_72 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_73 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_74 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_75 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_76 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_77 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_78 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_79 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"0008C00282606B77692B520002040000000000005100000008444AAAA8AB1422",
                   INITP_01 => X"C9451559B25D4D43459DB6B9B13B5FD2AC73B2EED4600CB72154A503B0000844",
                   INITP_02 => X"857FFFFFFFF44C48BC40B100000808D38A8542A00A10A895C2062A8C05805145",
                   INITP_03 => X"0000A0168019E400007007FFDB6E9C000ECA25400986345811A2A888D1448922",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(   ADDRARDADDR => address_a,
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a_l(31 downto 0),
                      DOPADOP => data_out_a_l(35 downto 32), 
                        DIADI => data_in_a(31 downto 0),
                      DIPADIP => data_in_a(35 downto 32), 
                          WEA => "0000",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b,
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b_l(31 downto 0),
                      DOPBDOP => data_out_b_l(35 downto 32), 
                        DIBDI => data_in_b_l(31 downto 0),
                      DIPBDIP => data_in_b_l(35 downto 32), 
                        WEBWE => we_b,
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0',
                   CASCADEINA => '0',
                   CASCADEINB => '0',
                INJECTDBITERR => '0',
                INJECTSBITERR => '0');
      --
      kcpsm6_rom_h: RAMB36E1
      generic map ( READ_WIDTH_A => 9,
                    WRITE_WIDTH_A => 9,
                    DOA_REG => 0,
                    INIT_A => X"000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 9,
                    WRITE_WIDTH_B => 9,
                    DOB_REG => 0,
                    INIT_B => X"000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    EN_ECC_READ => FALSE,
                    EN_ECC_WRITE => FALSE,
                    RAM_EXTENSION_A => "NONE",
                    RAM_EXTENSION_B => "NONE",
                    SIM_DEVICE => "VIRTEX6",
                    INIT_00 => X"1AE8EAC8EA100090E8B8B0B801000101B068480B0C0C10105858B8005B005800",
                    INIT_01 => X"0028B0D9D8C800100808091008080910080809288A28CAC88AF0CAC8CAC88A28",
                    INIT_02 => X"2C0C000C2803000300001C10001C28010001000100095B2849B06949695B285B",
                    INIT_03 => X"0900090009000900090009000900090009000900090009000900095B285B0000",
                    INIT_04 => X"E959A8E9000C0C28B0E38B0051105B0009000900090009000900090009000900",
                    INIT_05 => X"0B0B0C0C005B000B0B000C5B005B5B0C005B0C0C5B000B0B0009000900095BB0",
                    INIT_06 => X"90E990E9000C0C10000B0C005B005910000B0C28B088E85880EB5B0C000B0C00",
                    INIT_07 => X"E990E990E990E990E90C91E991E9000C0C90E990E990E990E990E990E90C90E9",
                    INIT_08 => X"B8000B0C000C000C000C000C000C000C000C000C0B000C000C0B2891E991E990",
                    INIT_09 => X"EEB8B7B6B8000C0C28BFBFBEBEBDBD000B0C0C10000B0C0C100100002C0B0B0C",
                    INIT_0A => X"90FFEE9A8A8971000A0A090009000100015B000C0C050588FBEB90FBEBB9B0FF",
                    INIT_0B => X"B5B5B4B4B3B1B8A8B8EFA0EFB8000B0C000C0C93835B0404B1B0F2E290E990E9",
                    INIT_0C => X"B4B4B3B0B0280D0D0EA8EE0FB0FAFAE95A5A28098FA0EF80EF3588FDEDB8B7B6",
                    INIT_0D => X"0D0B280FB8B4B4B3B5B5B80E10F08A8BB0E15290E9010C0C0B0AB0EEB0010D0D",
                    INIT_0E => X"0B0C0CB0E959B0010D0D0E06010B0C0C28B0E58D23B0C8A4A4A308C80000520D",
                    INIT_0F => X"00B0010D0D280D0D0EB0E9280D0D0EB0E9280D0D0EB0E9100EB0E9D000280D0D",
                    INIT_10 => X"6F91E78FB89C9C8B01B8B1520F2891B8FFEE9F8EB800B1E9B8B1B8280D0D0ED0",
                    INIT_11 => X"B6B5B5B8000C0C91EEB8B6B8B80101B811B80B0C0CB8250D0D01280F0E011191",
                    INIT_12 => X"0C01B80A0AB80C91EE000C0C91EE91EE0EB6B3B4B4010C010CB80A0A0E0B0C0C",
                    INIT_13 => X"7B7C1C91EB7C010A097C2C0CB801B8B1B1EC110000B1E80C00B1EC01B8100C01",
                    INIT_14 => X"00015BB8B411B8B1EC8C0151BC0C7949794979000C79000C0C78B801B87C0109",
                    INIT_15 => X"0C000B0C000C28B8B1DACA0191E991EE9C9C8B01B828B89B8B5B000100010001",
                    INIT_16 => X"0A0A0A0A0A0A0A0A0A0A0A0A119D8D8AA8E25388EA250A28B1FBEB000CA8E900",
                    INIT_17 => X"B0B828B87FEE9F8EB80A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A",
                    INIT_18 => X"0A0B0C0CB8000C0C000B1000B8B3B80C0C01100C0CE0E90B000C0CB00188E8B8",
                    INIT_19 => X"110C010CB80E0A0A0B0C0CB8000C0C000B0C0CC8FBEB000C0C110C010CB80E0A",
                    INIT_1A => X"00015BB2B2000C0CB3B3A86949D1FBEB000C0C91E991E9000C0C100B0C88E949",
                    INIT_1B => X"00095B100B0C000C0C9383B8B3B3B2B2B85BB169496BB8B3B30B0BB800090001",
                    INIT_1C => X"59010A09010A09010A09010A09B1F9E95959000109115B002901000900090009",
                    INIT_1D => X"01000100010009285BB1E1897100515B285B005B006959695969596959695969",
                    INIT_1E => X"01000100010001000100090106285B118991E971000900010001000100091000",
                    INIT_1F => X"00000000000000001009100910000928B169010001000100010009012801B169",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_40 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_41 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_42 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_43 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_44 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_45 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_46 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_47 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_48 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_49 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_50 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_51 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_52 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_53 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_54 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_55 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_56 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_57 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_58 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_59 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_60 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_61 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_62 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_63 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_64 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_65 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_66 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_67 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_68 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_69 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_70 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_71 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_72 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_73 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_74 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_75 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_76 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_77 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_78 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_79 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"552A5552A99F94890E588CAB299755555555555F2ADAAB4FE1111555575F83DF",
                   INITP_01 => X"E451455C1308C41C282C942C0448256403AC844A831561224C8118B04AAAA535",
                   INITP_02 => X"70FFFFFFFFF895954B9A1E6AB722B67ED64EBC4D654A0540195FC66748C70EB1",
                   INITP_03 => X"00ADB55ED5575AAB55CDFD554924372AB210E61AA4249524A81912540CBA649D",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(   ADDRARDADDR => address_a,
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a_h(31 downto 0),
                      DOPADOP => data_out_a_h(35 downto 32), 
                        DIADI => data_in_a(31 downto 0),
                      DIPADIP => data_in_a(35 downto 32), 
                          WEA => "0000",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b,
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b_h(31 downto 0),
                      DOPBDOP => data_out_b_h(35 downto 32), 
                        DIBDI => data_in_b_h(31 downto 0),
                      DIPBDIP => data_in_b_h(35 downto 32), 
                        WEBWE => we_b,
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0',
                   CASCADEINA => '0',
                   CASCADEINB => '0',
                INJECTDBITERR => '0',
                INJECTSBITERR => '0');
      --
    end generate v6;
    --
    --
    akv7 : if (C_FAMILY = "7S") generate
      --
      address_a <= '1' & address(11 downto 0) & "111";
      instruction <= data_out_a_h(32) & data_out_a_h(7 downto 0) & data_out_a_l(32) & data_out_a_l(7 downto 0);
      data_in_a <= "000000000000000000000000000000000000";
      jtag_dout <= data_out_b_h(32) & data_out_b_h(7 downto 0) & data_out_b_l(32) & data_out_b_l(7 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b_l <= "000" & data_out_b_l(32) & "000000000000000000000000" & data_out_b_l(7 downto 0);
        data_in_b_h <= "000" & data_out_b_h(32) & "000000000000000000000000" & data_out_b_h(7 downto 0);
        address_b <= "1111111111111111";
        we_b <= "00000000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b_h <= "000" & jtag_din(17) & "000000000000000000000000" & jtag_din(16 downto 9);
        data_in_b_l <= "000" & jtag_din(8) & "000000000000000000000000" & jtag_din(7 downto 0);
        address_b <= '1' & jtag_addr(11 downto 0) & "111";
        we_b <= jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      --
      kcpsm6_rom_l: RAMB36E1
      generic map ( READ_WIDTH_A => 9,
                    WRITE_WIDTH_A => 9,
                    DOA_REG => 0,
                    INIT_A => X"000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 9,
                    WRITE_WIDTH_B => 9,
                    DOB_REG => 0,
                    INIT_B => X"000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    EN_ECC_READ => FALSE,
                    EN_ECC_WRITE => FALSE,
                    RAM_EXTENSION_A => "NONE",
                    RAM_EXTENSION_B => "NONE",
                    SIM_DEVICE => "7SERIES",
                    INIT_00 => X"DF007B006119D909FF0010018B99C2E4114000000007090A0131FF3574317435",
                    INIT_01 => X"420039000001003980969839400D0339102700000A00F600072B1100E900B900",
                    INIT_02 => X"041F5F1E002061204349FB6149FB007043804390430054000444010508640074",
                    INIT_03 => X"084300430043FF43FF43FF4301430143A843C043804304430043005400744349",
                    INIT_04 => X"8000000452003900937001436006744302430143A843C043034302430143DC43",
                    INIT_05 => X"F40119000674931C16490C54231514045F130800749313004304430043005464",
                    INIT_06 => X"2800EF145228035F2D0801357431745F2D1001001000FF1C5F01261D5F041B23",
                    INIT_07 => X"1DCD1BCD1ACD18D11C0CFE0046175208032D13CD1DCD1BCD1ACD18D51C2C3817",
                    INIT_08 => X"005F0100CD28CD085CE85CC85CA85C885C685C48005C285C080800F7150E13CD",
                    INIT_09 => X"0400F0C001552826000000000000005F022C012D5F012C016160434904500004",
                    INIT_0A => X"62FF0400010130430000004338436043705455282870600000007900000046FF",
                    INIT_0B => X"B0A090807030010000FF87FF015F4001232C2890807450403058B0A067FF670A",
                    INIT_0C => X"908070B0A0002A02000080FE9B0A0D0201000000018E00D000A000000000F0C0",
                    INIT_0D => X"07000000019080701000000097AA0101B55040B400B907004015B480B5C9E202",
                    INIT_0E => X"000006E32005FBC9D4020020B900000700C1B00150C6010000060400211BA00D",
                    INIT_0F => X"BE97C9DA0200260203E07200340205F57300300203EF65E002E96297BE003402",
                    INIT_10 => X"0F22300100000001D30140E0000034001FFF0001012D13000030010005020497",
                    INIT_11 => X"70809001552824540000C00100F6E401340000FE0601A0270234000000F61414",
                    INIT_12 => X"2CAB00401F013463055528246103610400C0D0B0A0BA2CAB3400401F0000FF06",
                    INIT_13 => X"02007F860000B00900008000017D0080750274CDD574000C806E019001D52CBA",
                    INIT_14 => X"43705400205F00940E019B8034000D060C070B52030A5208000901AB0000B000",
                    INIT_15 => X"035F200123240000AC00019BB800B402000001B9010001000174438043904360",
                    INIT_16 => X"2F2054534F50002F20544547CA0001010060400000A00000BF20005520001752",
                    INIT_17 => X"1001000003FF000101006369736142203A6E6F6974617A69726F68747541005F",
                    INIT_18 => X"00800007015508245F022D5F0010010C01F75F0C015F010152001D2DF7000000",
                    INIT_19 => X"BA0CAB1400021E00C00007015508245F03001D00005B550826BA0CAB14000240",
                    INIT_1A => X"437054607055082870600002054010005508262702160152001D5F2201000003",
                    INIT_1B => X"4300345F4001230C285040007060504001746B07051000706010000143184360",
                    INIT_1C => X"39B0261DB03631B0312CB02C27AB0E801D00317D00AB74430230430043604300",
                    INIT_1D => X"704280429042030074B1400130433034007431F431403A203BE036C037A03880",
                    INIT_1E => X"F13FD04270428042904202F4200074CB01D1FF30420042704280429042033F42",
                    INIT_1F => X"00000000000000003F043F063F420500ED01F13F704280429042D8F400F6DF01",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_40 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_41 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_42 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_43 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_44 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_45 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_46 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_47 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_48 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_49 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_50 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_51 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_52 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_53 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_54 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_55 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_56 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_57 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_58 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_59 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_60 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_61 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_62 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_63 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_64 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_65 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_66 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_67 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_68 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_69 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_70 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_71 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_72 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_73 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_74 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_75 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_76 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_77 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_78 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_79 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"0008C00282606B77692B520002040000000000005100000008444AAAA8AB1422",
                   INITP_01 => X"C9451559B25D4D43459DB6B9B13B5FD2AC73B2EED4600CB72154A503B0000844",
                   INITP_02 => X"857FFFFFFFF44C48BC40B100000808D38A8542A00A10A895C2062A8C05805145",
                   INITP_03 => X"0000A0168019E400007007FFDB6E9C000ECA25400986345811A2A888D1448922",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(   ADDRARDADDR => address_a,
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a_l(31 downto 0),
                      DOPADOP => data_out_a_l(35 downto 32), 
                        DIADI => data_in_a(31 downto 0),
                      DIPADIP => data_in_a(35 downto 32), 
                          WEA => "0000",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b,
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b_l(31 downto 0),
                      DOPBDOP => data_out_b_l(35 downto 32), 
                        DIBDI => data_in_b_l(31 downto 0),
                      DIPBDIP => data_in_b_l(35 downto 32), 
                        WEBWE => we_b,
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0',
                   CASCADEINA => '0',
                   CASCADEINB => '0',
                INJECTDBITERR => '0',
                INJECTSBITERR => '0');
      --
      kcpsm6_rom_h: RAMB36E1
      generic map ( READ_WIDTH_A => 9,
                    WRITE_WIDTH_A => 9,
                    DOA_REG => 0,
                    INIT_A => X"000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 9,
                    WRITE_WIDTH_B => 9,
                    DOB_REG => 0,
                    INIT_B => X"000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    EN_ECC_READ => FALSE,
                    EN_ECC_WRITE => FALSE,
                    RAM_EXTENSION_A => "NONE",
                    RAM_EXTENSION_B => "NONE",
                    SIM_DEVICE => "7SERIES",
                    INIT_00 => X"1AE8EAC8EA100090E8B8B0B801000101B068480B0C0C10105858B8005B005800",
                    INIT_01 => X"0028B0D9D8C800100808091008080910080809288A28CAC88AF0CAC8CAC88A28",
                    INIT_02 => X"2C0C000C2803000300001C10001C28010001000100095B2849B06949695B285B",
                    INIT_03 => X"0900090009000900090009000900090009000900090009000900095B285B0000",
                    INIT_04 => X"E959A8E9000C0C28B0E38B0051105B0009000900090009000900090009000900",
                    INIT_05 => X"0B0B0C0C005B000B0B000C5B005B5B0C005B0C0C5B000B0B0009000900095BB0",
                    INIT_06 => X"90E990E9000C0C10000B0C005B005910000B0C28B088E85880EB5B0C000B0C00",
                    INIT_07 => X"E990E990E990E990E90C91E991E9000C0C90E990E990E990E990E990E90C90E9",
                    INIT_08 => X"B8000B0C000C000C000C000C000C000C000C000C0B000C000C0B2891E991E990",
                    INIT_09 => X"EEB8B7B6B8000C0C28BFBFBEBEBDBD000B0C0C10000B0C0C100100002C0B0B0C",
                    INIT_0A => X"90FFEE9A8A8971000A0A090009000100015B000C0C050588FBEB90FBEBB9B0FF",
                    INIT_0B => X"B5B5B4B4B3B1B8A8B8EFA0EFB8000B0C000C0C93835B0404B1B0F2E290E990E9",
                    INIT_0C => X"B4B4B3B0B0280D0D0EA8EE0FB0FAFAE95A5A28098FA0EF80EF3588FDEDB8B7B6",
                    INIT_0D => X"0D0B280FB8B4B4B3B5B5B80E10F08A8BB0E15290E9010C0C0B0AB0EEB0010D0D",
                    INIT_0E => X"0B0C0CB0E959B0010D0D0E06010B0C0C28B0E58D23B0C8A4A4A308C80000520D",
                    INIT_0F => X"00B0010D0D280D0D0EB0E9280D0D0EB0E9280D0D0EB0E9100EB0E9D000280D0D",
                    INIT_10 => X"6F91E78FB89C9C8B01B8B1520F2891B8FFEE9F8EB800B1E9B8B1B8280D0D0ED0",
                    INIT_11 => X"B6B5B5B8000C0C91EEB8B6B8B80101B811B80B0C0CB8250D0D01280F0E011191",
                    INIT_12 => X"0C01B80A0AB80C91EE000C0C91EE91EE0EB6B3B4B4010C010CB80A0A0E0B0C0C",
                    INIT_13 => X"7B7C1C91EB7C010A097C2C0CB801B8B1B1EC110000B1E80C00B1EC01B8100C01",
                    INIT_14 => X"00015BB8B411B8B1EC8C0151BC0C7949794979000C79000C0C78B801B87C0109",
                    INIT_15 => X"0C000B0C000C28B8B1DACA0191E991EE9C9C8B01B828B89B8B5B000100010001",
                    INIT_16 => X"0A0A0A0A0A0A0A0A0A0A0A0A119D8D8AA8E25388EA250A28B1FBEB000CA8E900",
                    INIT_17 => X"B0B828B87FEE9F8EB80A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A",
                    INIT_18 => X"0A0B0C0CB8000C0C000B1000B8B3B80C0C01100C0CE0E90B000C0CB00188E8B8",
                    INIT_19 => X"110C010CB80E0A0A0B0C0CB8000C0C000B0C0CC8FBEB000C0C110C010CB80E0A",
                    INIT_1A => X"00015BB2B2000C0CB3B3A86949D1FBEB000C0C91E991E9000C0C100B0C88E949",
                    INIT_1B => X"00095B100B0C000C0C9383B8B3B3B2B2B85BB169496BB8B3B30B0BB800090001",
                    INIT_1C => X"59010A09010A09010A09010A09B1F9E95959000109115B002901000900090009",
                    INIT_1D => X"01000100010009285BB1E1897100515B285B005B006959695969596959695969",
                    INIT_1E => X"01000100010001000100090106285B118991E971000900010001000100091000",
                    INIT_1F => X"00000000000000001009100910000928B169010001000100010009012801B169",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_40 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_41 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_42 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_43 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_44 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_45 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_46 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_47 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_48 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_49 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_50 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_51 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_52 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_53 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_54 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_55 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_56 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_57 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_58 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_59 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_60 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_61 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_62 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_63 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_64 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_65 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_66 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_67 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_68 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_69 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_70 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_71 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_72 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_73 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_74 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_75 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_76 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_77 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_78 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_79 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"552A5552A99F94890E588CAB299755555555555F2ADAAB4FE1111555575F83DF",
                   INITP_01 => X"E451455C1308C41C282C942C0448256403AC844A831561224C8118B04AAAA535",
                   INITP_02 => X"70FFFFFFFFF895954B9A1E6AB722B67ED64EBC4D654A0540195FC66748C70EB1",
                   INITP_03 => X"00ADB55ED5575AAB55CDFD554924372AB210E61AA4249524A81912540CBA649D",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(   ADDRARDADDR => address_a,
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a_h(31 downto 0),
                      DOPADOP => data_out_a_h(35 downto 32), 
                        DIADI => data_in_a(31 downto 0),
                      DIPADIP => data_in_a(35 downto 32), 
                          WEA => "0000",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b,
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b_h(31 downto 0),
                      DOPBDOP => data_out_b_h(35 downto 32), 
                        DIBDI => data_in_b_h(31 downto 0),
                      DIPBDIP => data_in_b_h(35 downto 32), 
                        WEBWE => we_b,
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0',
                   CASCADEINA => '0',
                   CASCADEINB => '0',
                INJECTDBITERR => '0',
                INJECTSBITERR => '0');
      --
    end generate akv7;
    --
  end generate ram_4k_generate;	              
  --
  --
  --
  --
  -- JTAG Loader
  --
  instantiate_loader : if (C_JTAG_LOADER_ENABLE = 1) generate
  --
    jtag_loader_6_inst : jtag_loader_6
    generic map(              C_FAMILY => C_FAMILY,
                       C_NUM_PICOBLAZE => 1,
                  C_JTAG_LOADER_ENABLE => C_JTAG_LOADER_ENABLE,
                 C_BRAM_MAX_ADDR_WIDTH => BRAM_ADDRESS_WIDTH,
	                  C_ADDR_WIDTH_0 => BRAM_ADDRESS_WIDTH)
    port map( picoblaze_reset => rdl_bus,
                      jtag_en => jtag_en,
                     jtag_din => jtag_din,
                    jtag_addr => jtag_addr(BRAM_ADDRESS_WIDTH-1 downto 0),
                     jtag_clk => jtag_clk,
                      jtag_we => jtag_we,
                  jtag_dout_0 => jtag_dout,
                  jtag_dout_1 => jtag_dout, -- ports 1-7 are not used
                  jtag_dout_2 => jtag_dout, -- in a 1 device debug 
                  jtag_dout_3 => jtag_dout, -- session.  However, Synplify
                  jtag_dout_4 => jtag_dout, -- etc require all ports to
                  jtag_dout_5 => jtag_dout, -- be connected
                  jtag_dout_6 => jtag_dout,
                  jtag_dout_7 => jtag_dout);
    --  
  end generate instantiate_loader;
  --
end low_level_definition;
--
--
-------------------------------------------------------------------------------------------
--
-- JTAG Loader 
--
-------------------------------------------------------------------------------------------
--
--
-- JTAG Loader 6 - Version 6.00
-- Kris Chaplin 4 February 2010
-- Ken Chapman 15 August 2011 - Revised coding style
--
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
--
library unisim;
use unisim.vcomponents.all;
--
entity jtag_loader_6 is
generic(              C_JTAG_LOADER_ENABLE : integer := 1;
                                  C_FAMILY : string := "V6";
                           C_NUM_PICOBLAZE : integer := 1;
                     C_BRAM_MAX_ADDR_WIDTH : integer := 10;
        C_PICOBLAZE_INSTRUCTION_DATA_WIDTH : integer := 18;
                              C_JTAG_CHAIN : integer := 2;
                            C_ADDR_WIDTH_0 : integer := 10;
                            C_ADDR_WIDTH_1 : integer := 10;
                            C_ADDR_WIDTH_2 : integer := 10;
                            C_ADDR_WIDTH_3 : integer := 10;
                            C_ADDR_WIDTH_4 : integer := 10;
                            C_ADDR_WIDTH_5 : integer := 10;
                            C_ADDR_WIDTH_6 : integer := 10;
                            C_ADDR_WIDTH_7 : integer := 10);
port(   picoblaze_reset : out std_logic_vector(C_NUM_PICOBLAZE-1 downto 0);
                jtag_en : out std_logic_vector(C_NUM_PICOBLAZE-1 downto 0) := (others => '0');
               jtag_din : out std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0) := (others => '0');
              jtag_addr : out std_logic_vector(C_BRAM_MAX_ADDR_WIDTH-1 downto 0) := (others => '0');
               jtag_clk : out std_logic := '0';
                jtag_we : out std_logic := '0';
            jtag_dout_0 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_1 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_2 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_3 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_4 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_5 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_6 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_7 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0));
end jtag_loader_6;
--
architecture Behavioral of jtag_loader_6 is
  --
  signal num_picoblaze       : std_logic_vector(2 downto 0);
  signal picoblaze_instruction_data_width : std_logic_vector(4 downto 0);
  --
  signal drck                : std_logic;
  signal shift_clk           : std_logic;
  signal shift_din           : std_logic;
  signal shift_dout          : std_logic;
  signal shift               : std_logic;
  signal capture             : std_logic;
  --
  signal control_reg_ce      : std_logic;
  signal bram_ce             : std_logic_vector(C_NUM_PICOBLAZE-1 downto 0);
  signal bus_zero            : std_logic_vector(C_NUM_PICOBLAZE-1 downto 0) := (others => '0');
  signal jtag_en_int         : std_logic_vector(C_NUM_PICOBLAZE-1 downto 0);
  signal jtag_en_expanded    : std_logic_vector(7 downto 0) := (others => '0');
  signal jtag_addr_int       : std_logic_vector(C_BRAM_MAX_ADDR_WIDTH-1 downto 0);
  signal jtag_din_int        : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal control_din         : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0):= (others => '0');
  signal control_dout        : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0):= (others => '0');
  signal control_dout_int    : std_logic_vector(7 downto 0):= (others => '0');
  signal bram_dout_int       : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0) := (others => '0');
  signal jtag_we_int         : std_logic;
  signal jtag_clk_int        : std_logic;
  signal bram_ce_valid       : std_logic;
  signal din_load            : std_logic;
  --
  signal jtag_dout_0_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_1_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_2_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_3_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_4_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_5_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_6_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_7_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal picoblaze_reset_int : std_logic_vector(C_NUM_PICOBLAZE-1 downto 0) := (others => '0');
  --        
begin
  bus_zero <= (others => '0');
  --
  jtag_loader_gen: if (C_JTAG_LOADER_ENABLE = 1) generate
    --
    -- Insert BSCAN primitive for target device architecture.
    --
    BSCAN_SPARTAN6_gen: if (C_FAMILY="S6") generate
    begin
      BSCAN_BLOCK_inst : BSCAN_SPARTAN6
      generic map ( JTAG_CHAIN => C_JTAG_CHAIN)
      port map( CAPTURE => capture,
                   DRCK => drck,
                  RESET => open,
                RUNTEST => open,
                    SEL => bram_ce_valid,
                  SHIFT => shift,
                    TCK => open,
                    TDI => shift_din,
                    TMS => open,
                 UPDATE => jtag_clk_int,
                    TDO => shift_dout);
    end generate BSCAN_SPARTAN6_gen;   
    --
    BSCAN_VIRTEX6_gen: if (C_FAMILY="V6") generate
    begin
      BSCAN_BLOCK_inst: BSCAN_VIRTEX6
      generic map(    JTAG_CHAIN => C_JTAG_CHAIN,
                    DISABLE_JTAG => FALSE)
      port map( CAPTURE => capture,
                   DRCK => drck,
                  RESET => open,
                RUNTEST => open,
                    SEL => bram_ce_valid,
                  SHIFT => shift,
                    TCK => open,
                    TDI => shift_din,
                    TMS => open,
                 UPDATE => jtag_clk_int,
                    TDO => shift_dout);
    end generate BSCAN_VIRTEX6_gen;   
    --
    BSCAN_7SERIES_gen: if (C_FAMILY="7S") generate
    begin
      BSCAN_BLOCK_inst: BSCANE2
      generic map(    JTAG_CHAIN => C_JTAG_CHAIN,
                    DISABLE_JTAG => "FALSE")
      port map( CAPTURE => capture,
                   DRCK => drck,
                  RESET => open,
                RUNTEST => open,
                    SEL => bram_ce_valid,
                  SHIFT => shift,
                    TCK => open,
                    TDI => shift_din,
                    TMS => open,
                 UPDATE => jtag_clk_int,
                    TDO => shift_dout);
    end generate BSCAN_7SERIES_gen;   
    --
    --
    -- Insert clock buffer to ensure reliable shift operations.
    --
    upload_clock: BUFG
    port map( I => drck,
              O => shift_clk);
    --        
    --        
    --  Shift Register      
    --        
    --
    control_reg_ce_shift: process (shift_clk)
    begin
      if shift_clk'event and shift_clk = '1' then
        if (shift = '1') then
          control_reg_ce <= shift_din;
        end if;
      end if;
    end process control_reg_ce_shift;
    --        
    bram_ce_shift: process (shift_clk)
    begin
      if shift_clk'event and shift_clk='1' then  
        if (shift = '1') then
          if(C_NUM_PICOBLAZE > 1) then
            for i in 0 to C_NUM_PICOBLAZE-2 loop
              bram_ce(i+1) <= bram_ce(i);
            end loop;
          end if;
          bram_ce(0) <= control_reg_ce;
        end if;
      end if;
    end process bram_ce_shift;
    --        
    bram_we_shift: process (shift_clk)
    begin
      if shift_clk'event and shift_clk='1' then  
        if (shift = '1') then
          jtag_we_int <= bram_ce(C_NUM_PICOBLAZE-1);
        end if;
      end if;
    end process bram_we_shift;
    --        
    bram_a_shift: process (shift_clk)
    begin
      if shift_clk'event and shift_clk='1' then  
        if (shift = '1') then
          for i in 0 to C_BRAM_MAX_ADDR_WIDTH-2 loop
            jtag_addr_int(i+1) <= jtag_addr_int(i);
          end loop;
          jtag_addr_int(0) <= jtag_we_int;
        end if;
      end if;
    end process bram_a_shift;
    --        
    bram_d_shift: process (shift_clk)
    begin
      if shift_clk'event and shift_clk='1' then  
        if (din_load = '1') then
          jtag_din_int <= bram_dout_int;
         elsif (shift = '1') then
          for i in 0 to C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-2 loop
            jtag_din_int(i+1) <= jtag_din_int(i);
          end loop;
          jtag_din_int(0) <= jtag_addr_int(C_BRAM_MAX_ADDR_WIDTH-1);
        end if;
      end if;
    end process bram_d_shift;
    --
    shift_dout <= jtag_din_int(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1);
    --
    --
    din_load_select:process (bram_ce, din_load, capture, bus_zero, control_reg_ce) 
    begin
      if ( bram_ce = bus_zero ) then
        din_load <= capture and control_reg_ce;
       else
        din_load <= capture;
      end if;
    end process din_load_select;
    --
    --
    -- Control Registers 
    --
    num_picoblaze <= conv_std_logic_vector(C_NUM_PICOBLAZE-1,3);
    picoblaze_instruction_data_width <= conv_std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1,5);
    --	
    control_registers: process(jtag_clk_int) 
    begin
      if (jtag_clk_int'event and jtag_clk_int = '1') then
        if (bram_ce_valid = '1') and (jtag_we_int = '0') and (control_reg_ce = '1') then
          case (jtag_addr_int(3 downto 0)) is 
            when "0000" => -- 0 = version - returns (7 downto 4) illustrating number of PB
                           --               and (3 downto 0) picoblaze instruction data width
                           control_dout_int <= num_picoblaze & picoblaze_instruction_data_width;
            when "0001" => -- 1 = PicoBlaze 0 reset / status
                           if (C_NUM_PICOBLAZE >= 1) then 
                            control_dout_int <= picoblaze_reset_int(0) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_0-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "0010" => -- 2 = PicoBlaze 1 reset / status
                           if (C_NUM_PICOBLAZE >= 2) then 
                             control_dout_int <= picoblaze_reset_int(1) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_1-1,5) );
                            else 
                             control_dout_int <= (others => '0');
                           end if;
            when "0011" => -- 3 = PicoBlaze 2 reset / status
                           if (C_NUM_PICOBLAZE >= 3) then 
                            control_dout_int <= picoblaze_reset_int(2) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_2-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "0100" => -- 4 = PicoBlaze 3 reset / status
                           if (C_NUM_PICOBLAZE >= 4) then 
                            control_dout_int <= picoblaze_reset_int(3) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_3-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "0101" => -- 5 = PicoBlaze 4 reset / status
                           if (C_NUM_PICOBLAZE >= 5) then 
                            control_dout_int <= picoblaze_reset_int(4) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_4-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "0110" => -- 6 = PicoBlaze 5 reset / status
                           if (C_NUM_PICOBLAZE >= 6) then 
                            control_dout_int <= picoblaze_reset_int(5) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_5-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "0111" => -- 7 = PicoBlaze 6 reset / status
                           if (C_NUM_PICOBLAZE >= 7) then 
                            control_dout_int <= picoblaze_reset_int(6) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_6-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "1000" => -- 8 = PicoBlaze 7 reset / status
                           if (C_NUM_PICOBLAZE >= 8) then 
                            control_dout_int <= picoblaze_reset_int(7) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_7-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "1111" => control_dout_int <= conv_std_logic_vector(C_BRAM_MAX_ADDR_WIDTH -1,8);
            when others => control_dout_int <= (others => '1');
          end case;
        else 
          control_dout_int <= (others => '0');
        end if;
      end if;
    end process control_registers;
    -- 
    control_dout(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-8) <= control_dout_int;
    --
    pb_reset: process(jtag_clk_int) 
    begin
      if (jtag_clk_int'event and jtag_clk_int = '1') then
        if (bram_ce_valid = '1') and (jtag_we_int = '1') and (control_reg_ce = '1') then
          picoblaze_reset_int(C_NUM_PICOBLAZE-1 downto 0) <= control_din(C_NUM_PICOBLAZE-1 downto 0);
        end if;
      end if;
    end process pb_reset;    
    --
    --
    -- Assignments 
    --
    control_dout (C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-9 downto 0) <= (others => '0') when (C_PICOBLAZE_INSTRUCTION_DATA_WIDTH > 8);
    --
    -- Qualify the blockram CS signal with bscan select output
    jtag_en_int <= bram_ce when bram_ce_valid = '1' else (others => '0');
    --      
    jtag_en_expanded(C_NUM_PICOBLAZE-1 downto 0) <= jtag_en_int;
    jtag_en_expanded(7 downto C_NUM_PICOBLAZE) <= (others => '0') when (C_NUM_PICOBLAZE < 8);
    --        
    bram_dout_int <= control_dout or jtag_dout_0_masked or jtag_dout_1_masked or jtag_dout_2_masked or jtag_dout_3_masked or jtag_dout_4_masked or jtag_dout_5_masked or jtag_dout_6_masked or jtag_dout_7_masked;
    --
    control_din <= jtag_din_int;
    --        
    jtag_dout_0_masked <= jtag_dout_0 when jtag_en_expanded(0) = '1' else (others => '0');
    jtag_dout_1_masked <= jtag_dout_1 when jtag_en_expanded(1) = '1' else (others => '0');
    jtag_dout_2_masked <= jtag_dout_2 when jtag_en_expanded(2) = '1' else (others => '0');
    jtag_dout_3_masked <= jtag_dout_3 when jtag_en_expanded(3) = '1' else (others => '0');
    jtag_dout_4_masked <= jtag_dout_4 when jtag_en_expanded(4) = '1' else (others => '0');
    jtag_dout_5_masked <= jtag_dout_5 when jtag_en_expanded(5) = '1' else (others => '0');
    jtag_dout_6_masked <= jtag_dout_6 when jtag_en_expanded(6) = '1' else (others => '0');
    jtag_dout_7_masked <= jtag_dout_7 when jtag_en_expanded(7) = '1' else (others => '0');
    --
    jtag_en <= jtag_en_int;
    jtag_din <= jtag_din_int;
    jtag_addr <= jtag_addr_int;
    jtag_clk <= jtag_clk_int;
    jtag_we <= jtag_we_int;
    picoblaze_reset <= picoblaze_reset_int;
    --        
  end generate jtag_loader_gen;
--
end Behavioral;
--
--
------------------------------------------------------------------------------------
--
-- END OF FILE m25p16_spi_uart_bridge.vhd
--
------------------------------------------------------------------------------------
